#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de los documentos.
# Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para listar, comentar, subir y descargar documentos
# * Sección financiaciones: se utiliza para listar, comentar, subir y descargar documentos
class DocumentoController < ApplicationController

  # Solo elimina cuando el documento pertenece
  before_filter :autorizar, :only => [ :eliminar, :modificar_crear ]
  before_filter :busca_etiquetas, :only => [ :anadir_filtro, :listado ]

	# en proyectos y en financiación: se redirecciona por defecto a ordenado
  def index
    #session[:documento_filtro] = etiqueta_todos 
    redirect_to :action => "ordenado"
  end

  # Hace un filtrado segun las etiquetas que tengamos seleccionadas
  def filtrado
    # Primero vaciamos la variable del filtros (hay una por seccion)
    session[("filtro_etiquetas_" + params[:seccion]).to_sym] = []
    # Incluimos el valor del selector si hay algo seleccionado 
    session[("filtro_etiquetas_" + params[:seccion]).to_sym] << params[:selector][:etiqueta_nueva] if params[:selector][:etiqueta_nueva].to_i != 0
    # Vamos incluyendo las etiquetas seleccionadas previamente
    if params[:filtro_etiqueta]
      params[:filtro_etiqueta].each_value do |id|
        session[("filtro_etiquetas_" + params[:seccion]).to_sym] << id if id != "" && id.to_i > 0
      end 
    end
    redirect_to :action => :listado
  end

  # Incluye la etiqueta seleccionada para poder filtrar
  def anadir_filtro
    render :update do |page|
      page.replace_html params[:update], :partial => "elemento_filtrado", :locals => { :indice => params[:indice].to_i, :valor => params[:selector][:etiqueta_nueva], :ultimo => true }
    end
  end

  # Elimina la etiqueta seleccionada de las elegidas para filtrar
  def quitar_filtro
    render :update do |page|
      page.replace_html params[:update], :inline => ''
    end
  end

	# en proyectos y en financiación: establece los parametros de ordenación
  def ordenado
    session[:documento_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:documento_orden] = params[:orden] ? params[:orden] : "adjunto_file_name" 
    redirect_to :action => "listado" if params[:seccion] == "proyectos"
    redirect_to :action => "listado_por_espacio" if params[:seccion] == "documentos"
    redirect_to :action => "listado" if params[:seccion] == "administracion"
  end

	# en proyectos y en financiación: lista los documentos disponibles
  def listado
    # Establece las condiciones para la busqueda de documentos segun sea de proyecto o global
    condiciones_proyecto = ( @proyecto ? "espacio.proyecto_id = " + @proyecto.id.to_s : "" ) 

    # Hace la busqueda de documentos segun tengamos filtros o no
    if session[("filtro_etiquetas_" + params[:seccion]).to_sym] and session[("filtro_etiquetas_" + params[:seccion]).to_sym].length > 0
      condicion_filtros = condiciones_proyecto + (condiciones_proyecto.empty? ? "" : " AND ") + " etiqueta_x_documento.etiqueta_id IN (?)", session[("filtro_etiquetas_" + params[:seccion]).to_sym]
      @documentos = Documento.paginate(:all, :include => ["etiqueta_x_documento", "espacio" ], :conditions => condicion_filtros, :group => ["documento.id"], :having => ["count(documento.id) >= " + session[("filtro_etiquetas_" + params[:seccion]).to_sym].length.to_s ], :order => (session[:documento_orden] + " " + session[:documento_asc_desc] ), :page => params[:page], :per_page => (session[:por_pagina] or 20) )
    else
      condicion_filtros = condiciones_proyecto unless params[:seccion] == "administracion"
      condicion_filtros = condiciones_proyecto + (condiciones_proyecto.empty? ? "" : " AND ") + " etiqueta.nombre LIKE ?", _("Exportacion") + " %" if params[:seccion] == "administracion"
      @documentos = Documento.paginate(:all, :include => [ "etiqueta_x_documento", "espacio", "etiqueta" ], :conditions => condicion_filtros, :order => (session[:documento_orden] + " " + session[:documento_asc_desc] ), :page => params[:page], :per_page => (session[:por_pagina] or 20) )
    end
  end
  

  
	# en proyectos y en financiación: prepara el formulario de edición o creación
  def editar_nuevo
    @documento = params[:id] ?  Documento.find(params[:id]) : Documento.new
    @espacios = Espacio.find(:all, :conditions => "proyecto_id is NULL and nombre != 'Proyectos' ").collect {|e| [e.nombre , e.id]}
    # Si estamos en administracion y no existen las etiquetas de exportacion las creamos
    [ _("Exportacion Presupuesto"), _("Exportacion Gasto"), ("Exportacion Transferencia") ].each { |etiqueta| Etiqueta.new(:nombre => etiqueta, :descripcion => etiqueta, :tipo => "proyecto").save if Etiqueta.find_by_nombre(etiqueta).nil? } if params[:seccion] == "administracion"
    @etiquetas_exportacion = Etiqueta.find(:all, :conditions => ["etiqueta.nombre LIKE ?", _("Exportacion") + " %"])
    @etiquetas_proyecto = Etiqueta.find(:all, :conditions => {"tipo" => "proyecto"})
    @etiquetas_comunes = Etiqueta.find(:all, :conditions => {"tipo" => "comunes"})
    @etiquetas_documento = @documento.etiqueta.collect{ |e| e.id }
    render :partial => "formulario"
  end

	# en proyectos y en financiación:  modifica o crea
  def modificar_crear
    @documento = params[:id] ?  Documento.find(params[:id]) : Documento.new
    # Solo ponemos el usuario_id si no existe o el documento es nuevo
    params[:documento][:usuario_id] = session[:usuario_identificado].id if !params[:id] || !@documento.usuario_id
    @documento.update_attributes params[:documento]
    @proyecto.espacio.documento << @documento if params[:seccion] == "proyectos"
    # Si estamos en administracion incluimos en documento en el espacio de plantillas de exportación
    if params[:seccion] == "administracion"
      espacio = Espacio.find_by_nombre "Plantillas exportación"
      espacio.documento << @documento
    end
    @documento.etiqueta.clear
    @documento.etiqueta_ids = params["etiqueta"].to_a.collect {|p| p[0] if p[1] == "1"} if @documento.errors.empty?
    # Si estamos trabajando en las fuentes de verificacion vinculamos el documento a la fuente de verificacion
    if params[:listado] == "listado_fuentes_verificacion" 
      logger.info ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><"
      FuenteVerificacionXDocumento.create(:documento_id => @documento.id, :fuente_verificacion_id => params[:fuente_verificacion_id])
    end
    # flash[:error] = @documento
    redirect_to :action => params[:listado] unless params[:listado] == "listado_fuentes_verificacion"
    redirect_to :action => params[:listado], :controller => "matriz" if params[:listado] == "listado_fuentes_verificacion"
  end    

	# en proyectos y en financiación: elimina documento
  def eliminar
    @documento = Documento.find(params[:id])
    @documento.destroy
    @proyecto if params[:seccion] == "proyectos"
    flash[:error_eliminar] = @documento
    redirect_to :action => 'listado'
  end

  def descargar
    @documento = Documento.find(params[:id])
      # file = File.new(@document.document.path, "r")
      # send_data(file.read,
      #  :filename     =>  @document.document_file_name,
      #  :type         =>  @document.document_content_type,
      #  :disposition  =>  'inline') 

    # Antes de descargar, comprobar que exista el fichero y marcar el error si no esta
    if File.exists?(@documento.adjunto.path)       
      send_file @documento.adjunto.path, :type => @documento.adjunto_content_type, :disposition => 'inline'
    else
      flash[:error] = _("No se pudo encontrar el documento. Contacte con el administrador del sistema.")
      redirect_to :action => 'listado'
    end
  end

  def seleccionar_espacio
    session[:espacio_seleccionado] = params[:id]
    redirect_to :action => :listado_por_espacio
  end


  def listado_por_espacio
    @espacio = Espacio.find(session[:espacio_seleccionado]) if session[:espacio_seleccionado]
    @documentos = Documento.paginate(:all, :conditions => {:espacio_id => session[:espacio_seleccionado]}, :page => params[:page], :per_page => (session[:por_pagina] or 20))
    @espacios = Espacio.find :all, :conditions => {"espacio_padre_id" => session[:espacio_seleccionado]}
  end


  # --
  ########## filtros ##########
  # ++
  private
    # Permitimos modificar y borrar solo al propietario del documento y al coordinador
    def autorizar
      if params[:id]
        @documento = Documento.find(params[:id])
        if @documento.usuario_id && @documento.usuario_id != session[:usuario_identificado].id && ((@proyecto && @proyecto.usuario_x_proyecto.find_by_usuario_id( session[:usuario_identificado].id ).rol != "coordinador") || session[:usuario_identificado].administracion )
          flash[:error] = ( _("Imposible: El documento no le pertenece.") )
          redirect_to :action => 'listado'
        end
      end
    end

    # Devuelve el conjunto de etiquetas valido para la seccion en la que estamos
    def busca_etiquetas
      condiciones_etiqueta = ""
      condiciones_etiqueta = "etiqueta.nombre LIKE ?", _("Exportacion") + " %" if params[:seccion] == "administracion"
      condiciones_etiqueta = "etiqueta.tipo = 'proyecto'" if params[:seccion] == "proyectos"
      @etiquetas = Etiqueta.find( :all, :conditions => condiciones_etiqueta ).collect{ |e| [e.nombre, e.id] }.sort{ |a, b| a[0] <=> b[0]}
    end

    # Devuelve el texto de seleccion de todas las etiquetas (o plantillas)
    def etiqueta_todos
      params[:seccion] == "administracion" ? _("Todas las plantillas") : _("Todas las etiquetas")
    end


end
