#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de Gastos. Este controlador es utilizado desde las secciones:
# * Sección agentes: se utiliza para la gestión de los gastos relacionados con agente.
#
class GastoAgentesController < ApplicationController

  # --
  # METODOS DE ENTRADA A PRESUPUESTO. Seleccion de ETAPA
  # ++

  # en agentes: se redirecciona por defecto a listado_etapas
  def index
    redirect_to :action => :filtrado_ordenado_iniciales
  end


  # --
  # METODOS DE GESTION DE Gasto: Listados de gasto, modificar_crear, y eliminar
  # ++

  def filtrado_ordenado_iniciales
    if @agente.etapa.empty? 
      flash[:error] = _("Tiene que definir por lo menos una etapa")
      redirect_to :controller => :datos_agente, :action => :etapas
    else
      session[:gasto_agentes_asc_desc] = "ASC" 
      session[:gasto_agentes_cadena_orden] = "fecha"
      session[:gasto_agentes_filtro_etapa] = "todas"
      session[:gasto_agentes_filtro_moneda] = "todas"
      session[:gasto_agentes_filtro_proyecto] = "todos"
      redirect_to :action => :listado
    end
  end

  # en agentes: establece los parametros de ordenación
  def ordenado
    session[:gasto_agentes_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:gasto_agentes_cadena_orden] = session[:gasto_agentes_orden] = params[:orden] ? params[:orden] : "fecha"
    session[:gasto_agentes_cadena_orden] = "partida.codigo" if session[:gasto_agentes_orden] == "partida.codigo_nombre"
    redirect_to :action => "listado"
  end

  def filtrado
    session[:gasto_agentes_filtro_etapa] = params[:filtro][:etapa]
    session[:gasto_agentes_filtro_moneda] = params[:filtro][:moneda]
    session[:gasto_agentes_filtro_proyecto] = params[:filtro][:proyecto]
    redirect_to :action => :listado
  end

 def elementos_filtrado
    filtro_etapa = [["Todas", "todas"]] + @agente.etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    filtro_proyecto =[["Todos", "todos"], ["No vinculado a proyectos", "no vinculado"]] + @agente.proyecto_implementador.collect{ |e| [e.nombre, e.id]}
    filtro_moneda =[["Todas", "todas"]] + @agente.moneda.collect{ |e| [e.nombre, e.id]}

    @opciones_filtrado = [{:rotulo =>  _("Selecciona Etapa: "), :nombre => "etapa", :opciones => filtro_etapa},
                          {:rotulo =>  _("Selecciona Moneda: "), :nombre => "moneda", :opciones => filtro_moneda},
                          {:rotulo =>  _("Selecciona Proyecto: "), :nombre => "proyecto", :opciones => filtro_proyecto}]
    texto_proyectos = if session[:gasto_agentes_filtro_proyecto] == "todos"
                        "Cualquier proyecto"
                      elsif session[:gasto_agentes_filtro_proyecto] == "no vinculado"
                        "No vinculado a proyectos"
                      else
                        Proyecto.find(session[:gasto_agentes_filtro_proyecto]).nombre
                      end
    @estado_filtrado = [ (session[:gasto_agentes_filtro_etapa] == "todas" ? "Cualquier etapa" : @etapa.nombre + " (" + @etapa.fecha_inicio.to_s + "/" + @etapa.fecha_fin.to_s + ")"), 
                         (session[:gasto_agentes_filtro_moneda] == "todas" ? "Cualquier moneda" : Moneda.find(session[:gasto_agentes_filtro_moneda]).nombre), texto_proyectos]
  end

  # en agentes: lista en función de la etapa y del agente
  def listado
    # en agentes: en caso de que la seccion sea agente hacemos una ultima transformacion a los elementos del listado para multipicarlos por el porcentaje.
    # Si el parametro select del find funcionase mejor no haria falta hacer a posteriori esto.
  
    @etapa = Etapa.find( session[:gasto_agentes_filtro_etapa]) unless session[:gasto_agentes_filtro_etapa] == "todas"
    condiciones = {:agente_id => @agente.id }
    condiciones["gasto.fecha"] = @etapa.fecha_inicio..@etapa.fecha_fin unless session[:gasto_agentes_filtro_etapa] == "todas"
    condiciones["gasto.moneda_id"] = session[:gasto_agentes_filtro_moneda] unless session[:gasto_agentes_filtro_moneda] == "todas"
    condiciones["gasto_x_proyecto.proyecto_id"] = session[:gasto_agentes_filtro_proyecto] unless session[:gasto_agentes_filtro_proyecto] == "todos" or session[:gasto_agentes_filtro_proyecto] == "no vinculado"
    condiciones["gasto_x_proyecto.proyecto_id"] = nil if session[:gasto_agentes_filtro_proyecto] == "no vinculado"
    @gastos = Gasto.paginate :all, :order => session[:gasto_agentes_cadena_orden] + " " + session[:gasto_agentes_asc_desc], 
                             :include => [:partida, :gasto_x_proyecto], :conditions => condiciones,
                             :page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:gasto_numero])
    elementos_filtrado

    @formato_xls = @gastos.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "gasto_agentes"
        @objetos = @gastos
        render 'comunes_xls/listado', :layout => false
      end
    end
  end

  # en agentes: prepara el formulario de edición o creación
  def editar_nuevo
    @gasto = @objeto = params[:id] ?  Gasto.find(params[:id]) : Gasto.new
    @etapa = Etapa.find( session[:gasto_agentes_filtro_etapa] ) unless session[:gasto_agentes_filtro_etapa] == "todas"
    @partidas = Partida.find(:all).collect {|a| [a.codigo_nombre, a.id]}
    @libros = @agente.libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}
    @monedas = @agente.libro.collect{|l| [l.moneda.abreviatura,l.moneda_id]}.uniq
    @proyectos = @gasto.gasto_x_proyecto @proyecto
    render :partial => "formulario"
  end

  # en agentes: modifica o crea comprobando que las fechas de la etapa coinciden
  def modificar_crear
    etapa = Etapa.find( session[:gasto_agentes_filtro_etapa] ) 
    gasto = params[:id] ?  Gasto.find(params[:id]) : Gasto.new(:agente_id => @agente.id)
    gasto.attributes = params[:gasto]
    libro = Libro.find_by_id(params[:pago][:libro_id]) if gasto.pago.count <= 1 && params[:pago] && params[:pago][:total] && params[:pago][:libro_id]
    params[:gasto][:moneda_id] = libro.moneda_id if libro
    gasto.update_attributes params[:gasto]
    gasto.actualizar_gasto_x_proyecto params[:proyectos] if gasto.errors.empty?
    gasto.gasto_x_proyecto.clear if params[:imputado_delegacion] == "1"
    # Genera un pago unico con fecha del gasto si se ha seleccionado y el gasto es nuevo
    if ( gasto.pago.count <= 1 && gasto.errors.empty? && params[:pago] && params[:pago][:total] == "1" )
      pago = gasto.pago.first || Pago.new
      pago.importe = gasto.importe
      pago.fecha = gasto.fecha
      pago.libro_id = params[:pago][:libro_id]
      pago.observaciones = params[:gasto][:concepto]
      pago.gasto = gasto
      pago.save
    end
    flash[:error] = gasto
    redirect_to :action => "ordenado"
  end

  # en agentes: elimina
  def eliminar
    @gasto = Gasto.find(params[:id])
    @gasto.destroy
    flash[:error_eliminar] = @gasto
    redirect_to :action => 'ordenado'
  end

  # --
  # METODOS AJAX DEL FORMULARIO: Moneda y subpartida.
  # ++

  # en agentes:  devuelve la moneda del libro seleccionado para el fomulario
  def moneda_libro
    @moneda = params[:id] != "" ? Libro.find(params[:id]).moneda : nil
    render :inline => <<-FIN
      <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
      <%= hidden_field('gasto', 'moneda_id', :value => @moneda.id) if @moneda%>
    FIN
  end

  # en agentes: devuelve las subpartidas asociadas a la partida seleccionada para el fomulario
  def subpartida
    render :partial => "subpartida", :locals => { :partida_id => params[:id]}
  end

  # --
  # METODOS AJAX DEL FORMULARIO PARA PROYECTOS
  # ++

  # en agentes: muestra el formulario para añadir proyectos
  def proyectos
    @gasto = @objeto = Gasto.find(params[:id])
    @proyectos = @gasto.gasto_x_proyecto @proyecto
    render :partial => "proyectos"
  end

  # en agentes: guarda los proyectos asociados a gasto.
  def guardar_proyectos
    @gasto = @objeto = Gasto.find(params[:id])
    @gasto.actualizar_gasto_x_proyecto params[:proyectos]
    render :update do |page|
      page.replace 'formulario', :inline => '<%= mensaje_error(@gasto) %><br>'
      #page.visual_effect :highlight, @gasto.id.to_s, :duration => 6
    end

  end

  # en agentes: añade un proyecto al formulario
  def anadir_proyecto
    render :update do |page|
      page.replace "proyecto_" + params[:linea], :partial => "proyecto" , :locals =>  { :linea => params[:linea].to_i, :ultima => true}
      page.call("Modalbox.resizeToContent")
    end
  end

end
