#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de la matriz. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para los objetivos especificos, resultados, actividades...

class MatrizController < ApplicationController
  
  before_filter :verificar_estado_formulacion, :only => [	:index, :modificar_crear_objetivo_especifico, :eliminar_objetivo_especifico,
								:modificar_crear_resultado, :eliminar_resultado,
								:crear_modificar_variable, :eliminar_variable]

  before_filter :verificar_estado_formulacion_ajax, :only => [	:crear_modificar_fuente_verificacion_objetivo_especifico, :crear_modificar_fuente_verificacion_resultado,
								:crear_modificar_indicador_objetivo_especifico, :crear_modificar_indicador_resultado,
								:eliminar_fuente_verificacion_objetivo_especifico, :eliminar_fuente_verificacion_resultado,
								:eliminar_indicador_objetivo_especifico, :eliminar_indicador_resultado,
								:crear_modificar_hipotesis_objetivo_especifico, :crear_modificar_hipotesis_resultado,
								:eliminar_hipotesis_objetivo_especifico, :eliminar_hipotesis_resultado,
								:crear_modificar_actividad, :eliminar_actividad ]

  before_filter :verificar_estado_ejecucion_ajax, :only => [	:crear_modificar_valor_variable, :eliminar_valor_variable ]

  def verificar_estado_formulacion
    #unless @proyecto.estado_actual and @proyecto.estado_actual.definicion_estado.formulacion
    unless @permitir_formulacion
      flash[:error] = _("El proyecto se encuentra 'sin estado'. En este estado no se pueden modificar la matriz")    if @proyecto.estado_actual.nil?
      flash[:error] = _("El proyecto se encuentra en estado de ") + @proyecto.estado_actual.definicion_estado.nombre + _(". En este estado no se puede modificar la matriz. No ha sido definido como 'estado de formulacion' por su administrador.") unless @proyecto.estado_actual.nil?
      redirect_to :action => "listado"
    end
  end

  def verificar_estado_formulacion_ajax
    #unless @proyecto.estado_actual and @proyecto.estado_actual.definicion_estado.formulacion
    unless @permitir_formulacion
      render :update do |page|
        page.replace_html "MB_content", :inline => "<%= mensaje_error ('El proyecto se encuentra 'sin estado'. En este estado no se pueden modificar los datos del proyecto.')  %>"  if @proyecto.estado_actual.nil?
          page.replace_html "MB_content", :inline => "<%= mensaje_error ('El proyecto se encuentra en estado de ' + @proyecto.estado_actual.definicion_estado.nombre + '. En este estado no se puede modificar la matriz. No ha sido definido como estado de formulacion por su administrador.')  %>" unless @proyecto.estado_actual.nil?
        page.call("Modalbox.resizeToContent")
      end 
    end
  end

  def verificar_estado_ejecucion_ajax
    unless @permitir_ejecucion
      render :update do |page|
        page.replace_html "MB_content", :inline => "<%= mensaje_error ('El proyecto se encuentra 'sin estado'. En este estado no se pueden modificar los datos del proyecto.')  %>"  if @proyecto.estado_actual.nil?
          page.replace_html "MB_content", :inline => "<%= mensaje_error ('El proyecto se encuentra en estado de ' + @proyecto.estado_actual.definicion_estado.nombre + '. En este estado no se pueden modificar los datos. No ha sido definido como estado de ejecución por su administrador.')  %>" unless @proyecto.estado_actual.nil?
        page.call("Modalbox.resizeToContent")
      end
    end
  end


  #--
  # OBJETIVOS ESPECIFICOS
  #--
	# en proyectos: se redirecciona por defecto a listado
  def index
    redirect_to :action => "listado"
  end

	# en proyectos: lista los objetivos específicos
  def listado
    @objetivos_especificos = @proyecto.objetivo_especifico.sort! {|x, y|  x.codigo <=> y.codigo }
    @resumen = {:url => {:action => :matriz, :controller => :resumen_proyecto, :sin_layout => true}}
    @formato_xls = 0
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "objetivo_especifico"
        @objetos = @objetivos_especificos
        @subobjetos = [ "indicador","fuente_verificacion","hipotesis" ]
        render 'comunes_xls/listado', :layout => false
      end
    end
  end

	# listado filtrado por pais y etapa
  def listado_filtrado
    @actividades = 
    @paises =  @proyecto.pais.collect{ |p| [p.nombre, p.id] }.sort{ |a, b| a[0] <=> b[0]}
    @etapas = @proyecto.etapa.collect{ |e| [e.nombre, e.id] }
    if params[:selector]
      @pais = Pais.find(params[:selector][:pais])
      @etapa = Etapa.find(params[:selector][:etapa])
      @condiciones = condiciones =  { "proyecto_id" =>  @proyecto.id, "actividad_x_pais.pais_id" => params[:selector][:pais], "actividad_x_etapa.etapa_id" => params[:selector][:etapa]}
      # [[objetivo, [resultado1,actividad1, act2...], [res2, act3, act4...]], [objetivo2, [...]]
      @objetivos_especificos = Array.new
      ObjetivoEspecifico.all.each do |objetivo|
        @resultados = Array.new
        objetivo.resultado.all.each do |resultado|
          @actividades = resultado.actividad.find(:all, :include => [:actividad_x_pais, :actividad_x_etapa], :conditions => condiciones )
          if @actividades
            @resultados += [[ resultado, @actividades ]]
          end
        end
        if @resultados
          @objetivos_especificos += [[objetivo, @resultados]]
        end
      end
    end
  end

  # selecciona el objetivo especifico de la matriz para la gestión de resultados y actividades.
  def seleccionar_objetivo_especifico
    session[:objetivo_especifico] = ObjetivoEspecifico.find(params[:objetivo_especifico_id])
    redirect_to :action => "listado_objetivo_especifico"
  end

	# en proyectos: lista los resultados de un objeto específico
  def listado_objetivo_especifico
    @resumen = {:url => {:action => :matriz, :controller => :resumen_proyecto, :sin_layout => true}, :mensaje => "Ver resumen de matriz"}
    @objetivo_especifico = session[:objetivo_especifico]
    @resultados = @objetivo_especifico.resultado.reload .sort! {|x, y|  x.codigo <=> y.codigo }
  end

	# en proyectos: prepara el formulario de edición o creación de un objeto específico
  def editar_nuevo_objetivo_especifico
    @objetivo_especifico = params[:id] ?  ObjetivoEspecifico.find(params[:id]) : nil
    render :partial => "objetivo_especifico"
  end

	# en proyectos: modifica o crea un objeto específico
  def modificar_crear_objetivo_especifico
    @objetivo_especifico = params[:id] ?  ObjetivoEspecifico.find(params[:id]) : ObjetivoEspecifico.new
    @objetivo_especifico.update_attributes params[:objetivo_especifico]
    @proyecto.objetivo_especifico << @objetivo_especifico
    flash[:error] = @objetivo_especifico
    redirect_to :action => "listado"
  end

	# en proyectos: elimina un objeto específico
  def eliminar_objetivo_especifico
    @objetivo_especifico = ObjetivoEspecifico.find(params[:id])
    @objetivo_especifico.destroy
    flash[:error_eliminar] = @objetivo_especifico
    redirect_to :action => "listado"
  end

  #--
  # CRONOGRAMA
  #++

  # Cronograma de actividades
  def cronograma
    @etapas = @proyecto.etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    if params[:selector]
      @etapa = Etapa.find(params[:selector][:etapa])
      @actividades = @etapa.actividad.sort! {|x, y|  x.codigo <=> y.codigo }

      respond_to do |format|
        format.html unless params[:selector][:salida] == 'fichero'
        format.xls do
          render 'comunes_xls/cronograma', :layout => false
        end if params[:selector][:salida] == 'fichero'
      end
    end
  end

  # Activa/desactiva el mes de la etapa para la actividad elegida
  def mes_cronograma
    if actividad_detallada = ActividadDetallada.find_by_mes(params[:mes], :conditions => { :etapa_id => params[:etapa_id], :actividad_id => params[:actividad_id], :seguimiento => !params[:seguimiento].nil? })
     actividad_detallada.destroy 
     seleccionado = false 
    else
     ActividadDetallada.new(:mes => params[:mes], :etapa_id => params[:etapa_id], :actividad_id => params[:actividad_id], :seguimiento => !params[:seguimiento].nil?).save
     seleccionado = true 
    end
    render :update do |page|
      page.replace params[:update], :partial => "mes_cronograma", :locals => { :seguimiento => params[:seguimiento], :seleccionado => seleccionado, :mes => params[:mes], :actividad_id => params[:actividad_id], :etapa_id => params[:etapa_id], :clickable => true }
    end
  end

  #--
  # RESULTADOS
  #++
	# en proyectos: prepara el formulario de edición o creación de un resultado
  def editar_nuevo_resultado
    @resultado = params[:id] ?  Resultado.find(params[:id]) : nil
    @objetivo_especifico_id = params[:objetivo_especifico_id]
    render :partial => "resultado"
  end

	# en proyectos: modifica o crea un resultado
  def modificar_crear_resultado
    @resultado = params[:id] ?  Resultado.find(params[:id]) : Resultado.new
    @resultado.proyecto_id = @proyecto.id
    @resultado.objetivo_especifico_id = session[:objetivo_especifico].id
    @resultado.update_attributes params[:resultado]
    flash[:error] = @resultado
    redirect_to :action => "listado_objetivo_especifico", :objetivo_especifico_id => params[:objetivo_especifico_id]
  end

	# en proyectos: elimina un resultado
  def eliminar_resultado
    @resultado = Resultado.find(params[:id])
    @resultado.destroy
    flash[:error_eliminar] = @resultado
    redirect_to :action => "listado_objetivo_especifico", :objetivo_especifico_id => params[:objetivo_especifico_id]
  end

  #--
  # ACTIVIDADES
  #++
	# en proyectos: lista las actividades de un resultado
  def actividades
    @actividades = Resultado.find(params[:resultado_id]).actividad.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html params[:update], :partial => "actividades"
    end
  end

	# en proyectos: prepara el formulario de edición o creación de una actividad
  def nueva_editar_actividad
    @actividad = params[:id] ? Actividad.find(params[:id]) : Actividad.new
    @paises = @proyecto.pais #.collect{ |p| [p.nombre, p.id, @actividad.pais.include?(p)? true : false ] }
    @etapas = @proyecto.etapa
    render :partial => "formulario_actividad", :locals => { :resultado_id => params[:resultado_id], :update => params[:update]}
  end

	# en proyectos: modifica o crea una actividad
  def crear_modificar_actividad
    @actividad = params[:id] ? Actividad.find(params[:id]) : Actividad.new
    @resultado = Resultado.find(params[:actividad][:resultado_id]) 
    @objetivo_especifico = ObjetivoEspecifico.find(params[:objetivo_especifico]) if params[:objetivo_especifico]
    @objetivo_especifico = ObjetivoEspecifico.find(params[:objetivo_especifico_id]) if params[:objetivo_especifico_id]
    params[:actividad][:proyecto_id] =  @proyecto.id
    @actividad.update_attributes params[:actividad]
    @actividad.pais_ids = params["pais"].to_a.collect {|p| p[0] if p[1] == "1"} if @actividad.errors.empty?
    @actividad.etapa_ids = params["etapa"].to_a.collect {|p| p[0] if p[1] == "1"} if @actividad.errors.empty?

    @actividades =  Resultado.find(params[:actividad][:resultado_id]).actividad.sort! {|x, y|  x.codigo <=> y.codigo }
    params[:resultado_id] = @resultado.id
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@actividad) %><br>'
      page.replace_html params[:update], :partial => "actividades" unless /matriz_filtrado_/.match(params[:update])
      page.replace_html params[:update], :partial => "actividades_filtrado", :locals => {:i => /(matriz_filtrado_)(\d*)/.match(params[:update])[2] }  if /matriz_filtrado_/.match(params[:update])
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end


	# en proyectos: elimina una actividad
  def eliminar_actividad
    @actividad = Actividad.find(params[:id])
    @actividad.destroy
    @actividades = Resultado.find(params[:resultado_id]).actividad.sort! {|x, y|  x.codigo <=> y.codigo }
    # flash[:error] = @actividad
    if @actividad.errors.count > 0
      @mensaje_error = ""
      @actividad.errors.each do |attr_name, message|
        @mensaje_error << message + "<br>"
      end
    else
      @mensaje = @proyecto
    end
    render :update do |page|
      page.replace_html params[:update], :partial => "actividades" unless /matriz_filtrado_/.match(params[:update])
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@actividad, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

        # en proyectos: modifica la fecha de una actividad
  def editar_fecha_actividad
    @actividad = Actividad.find_by_id params[:id]
    render :partial => "formulario_fecha_actividad"
  end

  def modificar_fecha_actividad
    @actividad = Actividad.find_by_id params[:id]
    @actividad.update_attributes params[:actividad]
    redirect_to :action => 'cronograma'
  end

  #--
  # INDICADORES, FUENTES DE VERIFICACION Y HIPOTESIS
  #++
	# en proyectos: lista los indicadores y las fuentes de un objetivo específico
  def indicadores_fuentes_objetivo_especifico
    @indicadores = ObjetivoEspecifico.find( params[:objetivo_especifico_id] ).indicador.sort! {|x, y|  x.codigo <=> y.codigo } 
    @fuentes_verificacion = ObjetivoEspecifico.find( params[:objetivo_especifico_id] ).fuente_verificacion.sort! {|x, y|  x.codigo <=> y.codigo }
    @hipotesis = Hipotesis.find(:all, :conditions => { :objetivo_especifico_id => params[:objetivo_especifico_id]} )
    render :update do |page|
      page.replace_html params[:update], :partial => "indicadores_fuentes", :locals => {:objetivo_especifico_id => params[:objetivo_especifico_id], :update => params[:update] }
    end    
  end

	# en proyectos: lista los indicadores y las fuentes de un resultado
  def indicadores_fuentes_resultado
    @indicadores = Resultado.find( params[:resultado_id] ).indicador.sort! {|x, y|  x.codigo <=> y.codigo }
    @fuentes_verificacion = Resultado.find( params[:resultado_id] ).fuente_verificacion.sort! {|x, y|  x.codigo <=> y.codigo }
    @hipotesis = Hipotesis.find(:all, :conditions => { :resultado_id => params[:resultado_id]} )

    render :update do |page|
      page.replace_html params[:update], :partial => "indicadores_fuentes", :locals => {:resultado_id => params[:resultado_id], :update => params[:update] }
    end    
  end

  #--
  # INDICADORES
  #++
	# en proyectos: prepara el formulario de edición o creación de un indicador para un objeto específico
  def nuevo_editar_indicador_objetivo_especifico
    @indicador = params[:id] ? Indicador.find(params[:id]) : Indicador.new
    render :partial => "formulario_indicador", :locals => { :objeto_id => params[:objetivo_especifico_id], :update => params[:update] }
  end

	# en proyectos: prepara el formulario de edición o creación de un indicador para un resultado
  def nuevo_editar_indicador_resultado
    @indicador = params[:id] ? Indicador.find(params[:id]) : Indicador.new
    render :partial => "formulario_indicador", :locals => { :objeto_id => params[:resultado_id], :update => params[:update] }
  end

	# en proyectos: modifica o crea un indicador para un objeto específico
  def crear_modificar_indicador_objetivo_especifico
    @indicador = params[:id] ? Indicador.find(params[:id]) : Indicador.new
    @indicador.update_attributes params[:indicador]
    ObjetivoEspecifico.find(params[:objetivo_especifico_id]).indicador  << @indicador if @indicador.errors.empty?
    @indicadores =  ObjetivoEspecifico.find(params[:objetivo_especifico_id]).indicador.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@indicador) %>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "indicadores", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

	# en proyectos: modifica o crea un indicador para un resultado
  def crear_modificar_indicador_resultado
    @indicador = params[:id] ? Indicador.find(params[:id]) : Indicador.new
    @indicador.update_attributes params[:indicador]
    Resultado.find(params[:resultado_id]).indicador  << @indicador if @indicador.errors.empty?
    @indicadores =  Resultado.find(params[:resultado_id]).indicador.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@indicador) %>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "indicadores", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

	# en proyectos: elimina un indicador para un objeto específico
  def eliminar_indicador_objetivo_especifico
    @objeto = Indicador.find(params[:id])
    @objeto.destroy
    @indicadores = ObjetivoEspecifico.find(params[:objetivo_especifico_id]).indicador.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html params[:update], :partial => "indicadores", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

	# en proyectos: elimina un indicador para un resultado
  def eliminar_indicador_resultado
    @objeto = Indicador.find(params[:id])
    @objeto.destroy
    @indicadores = Resultado.find(params[:resultado_id]).indicador.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html params[:update], :partial => "indicadores", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update], :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

  #--
  # VARIABLES DE INDICADORES
  #++

	# en proyectos: listado de variables de indicadores
  def variables
    # Todos los indicadores
    indicadores = Indicador.find(:all,:include=>["objetivo_especifico","resultado"],:conditions=>["objetivo_especifico.proyecto_id=? OR resultado.proyecto_id=?",@proyecto.id,@proyecto.id])
    # Segun de donde se venga, se recoge el indicador elegido (puede venir desde el filtrado o desde el listado de indicadores)
    @indicador_id = params[:indicador_id] if params[:indicador_id] && !params[:indicador_id].nil?
    @indicador_id = params[:filtro][:indicador] if params[:filtro] && params[:filtro][:indicador] != "todos"

    # Sacamos las variables segun tengamos filtro de indicador o no
    if @indicador_id
      indicador = Indicador.find_by_id(@indicador_id)
      vars = indicador.variable_indicador
    # Esto lo hacemos asi para sacar las variables relacionadas con indicadores de objetivos especificos y de resultados
    else
      vars = []
      indicadores.each { |i| vars += i.variable_indicador }
    end

    # Establece las variables para los filtros
    session[:matriz_filtro_indicador] = @indicador_id || "todos"
    @opciones_filtrado = [ {:rotulo =>  _("Selecciona Indicador: "), :nombre => "indicador", :opciones => [["Todos", "todos"]]  + indicadores.collect { |i| [i.codigo_completo, i.id] } } ]
    @estado_filtrado = [ indicador.nil? ? "Cualquier Indicador" : "Indicador: " + indicador.codigo_completo ]

    # Hace la seleccion final de elementos ordenados por codigo y nombre
    @variables = vars.sort! {|x,y| (x.indicador.codigo_completo + x.nombre) <=> (y.indicador.codigo_completo + y.nombre) }

    # Volcado html o xls
    @formato_xls = 0 
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "variable_indicador"
        @objetos = @variables
        @subobjetos = [ "valor_medido" ]
        render 'comunes_xls/listado', :layout => false
      end
    end
  end

	# en proyectos: prepara el formulario de edicion o creacion de una variable de indicador
  def editar_variable
    @variable = params[:id] ? VariableIndicador.find(params[:id]) : VariableIndicador.new(:indicador_id => params[:indicador_id])
    @indicadores = Indicador.find(:all,:include=>["objetivo_especifico","resultado"],:conditions=>["objetivo_especifico.proyecto_id=? OR resultado.proyecto_id=?",@proyecto.id,@proyecto.id]).collect { |i| [i.codigo_completo, i.id] }
    if @variable.id
      @valor_base = @variable.valor_base
      @valor_objetivo = @variable.valor_objetivo
    end 
    render :partial => "formulario_variable"
  end

	# en proyectos modifica o crea una variable de indicador
  def crear_modificar_variable
    @variable = params[:id] ? VariableIndicador.find(params[:id]) : VariableIndicador.new
    @variable.update_attributes params[:variable]
    if @variable.errors.empty?
      valor_base = @variable.valor_base || ValorVariableIndicador.new
      valor_base.update_attributes params[:valor_base]
      @variable.valor_base = valor_base

      valor_objetivo = @variable.valor_objetivo || ValorVariableIndicador.new
      valor_objetivo.update_attributes params[:valor_objetivo]
      @variable.valor_objetivo = valor_objetivo

      @variable.save
    end
    flash[:error] = @variable
    redirect_to :action => "variables", :indicador_id => params[:indicador_id]
  end

	# en proyectos: elimina la variable de un indicador
  def eliminar_variable
    @objeto = VariableIndicador.find(params[:id])
    @objeto.destroy
    flash[:error_eliminar] = @objeto
    redirect_to :action => "variables", :indicador_id => params[:indicador_id] 
  end

	# en proyectos, presenta un sublistado de valores intermedios de variables
  def valores_variable
    variable = VariableIndicador.find(params[:variable_id])
    @valores = variable.valor_medido.find(:all,:order => "fecha")
    render :update do |page|
      page.replace_html params[:update], :partial => "valores_variable"
    end
  end

  def nuevo_editar_valor_variable
    @valor_variable = params[:id] ? ValorVariableIndicador.find(params[:id]) : ValorVariableIndicador.new(:variable_indicador_id => params[:variable_id])
    render :partial => "formulario_valor_variable"
  end

  def crear_modificar_valor_variable
    @valor_variable = params[:id] ? ValorVariableIndicador.find(params[:id]) : ValorVariableIndicador.new
    @valor_variable.update_attributes params[:valor_variable]
    @valores = VariableIndicador.find(params[:variable_id]).valor_medido.find(:all, :order => "fecha")
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@valor_variable) %>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "valores_variable"
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

  def eliminar_valor_variable
    @objeto = ValorVariableIndicador.find(params[:id])
    @objeto.destroy
    @valores = VariableIndicador.find(params[:variable_id]).valor_medido.find(:all, :order => "fecha")
    render :update do |page|
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "valores_variable"
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

  #--
  # FUENTES DE VERIFICACION
  #++
	# en proyectos: prepara el formulario de edición o creación de una fuente de verificación para un objeto específico
  def nueva_editar_fuente_verificacion_objetivo_especifico
    @fuente_verificacion = params[:id] ? FuenteVerificacion.find(params[:id]) : FuenteVerificacion.new
    # En cuarentena porque a veces parecia que fallaba
    #@indicadores = Indicador.find(:all, :conditions => { :objetivo_especifico_id => params[:objetivo_especifico_id] } ).collect{ |i| [i.codigo, i.id] }.sort! {|x, y|  x.codigo <=> y.codigo }
    @indicadores = Indicador.find(:all, :conditions => { :objetivo_especifico_id => params[:objetivo_especifico_id] } ).collect{ |i| [i.codigo, i.id] }
    render :partial => "formulario_fuente_verificacion", :locals => { :fuente_id => params[:id], :objetivo_especifico_id => params[:objetivo_especifico_id], :update => params[:update] }
  end

	# en proyectos: prepara el formulario de edición o creación de una fuente de verificación para un resultado
  def nueva_editar_fuente_verificacion_resultado
    @fuente_verificacion = params[:id] ? FuenteVerificacion.find(params[:id]) : FuenteVerificacion.new
    # En cuarentena porque a veces parecia que fallaba
    #@indicadores = Indicador.find(:all, :conditions => { :resultado_id => params[:resultado_id] } ).collect{ |i| [i.codigo, i.id] }.sort! {|x, y|  x.codigo <=> y.codigo }
    @indicadores = Indicador.find(:all, :conditions => { :resultado_id => params[:resultado_id] } ).collect{ |i| [i.codigo, i.id] }
    render :partial => "formulario_fuente_verificacion", :locals => { :fuente_id => params[:id], :resultado_id => params[:resultado_id], :update => params[:update] }
  end

	# en proyectos: modifica o crea una fuente de verificación para un objeto específico
  def crear_modificar_fuente_verificacion_objetivo_especifico
    params['fuente_verificacion'][:objetivo_especifico_id] = params[:objetivo_especifico_id]
    @fuente_verificacion = params[:id] ? FuenteVerificacion.find(params[:id]) : FuenteVerificacion.new
    @fuente_verificacion.update_attributes params[:fuente_verificacion]
    ObjetivoEspecifico.find(params[:objetivo_especifico_id]).fuente_verificacion  << @fuente_verificacion if @fuente_verificacion.errors.empty?
    @fuentes_verificacion =  ObjetivoEspecifico.find(params[:objetivo_especifico_id]).fuente_verificacion.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@fuente_verificacion) %>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "fuentes_verificacion", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

	# en proyectos: modifica o crea una fuente de verificación para un resultado
  def crear_modificar_fuente_verificacion_resultado
    params['fuente_verificacion'][:resultado_id] = params[:resultado_id] if params[:resultado_id]
    @fuente_verificacion = params[:id] ? FuenteVerificacion.find(params[:id]) : FuenteVerificacion.new
    @fuente_verificacion.update_attributes params[:fuente_verificacion]
    Resultado.find(params[:resultado_id]).fuente_verificacion  << @fuente_verificacion if @fuente_verificacion.errors.empty?
    @fuentes_verificacion =  Resultado.find(params[:resultado_id]).fuente_verificacion.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@fuente_verificacion) %>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "fuentes_verificacion", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

	# en proyectos: elimina una fuente de verificación para un objeto específico
  def eliminar_fuente_verificacion_objetivo_especifico
    @objeto = FuenteVerificacion.find(params[:id])
    @objeto.destroy
    @fuentes_verificacion = ObjetivoEspecifico.find(params[:objetivo_especifico_id]).fuente_verificacion.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "fuentes_verificacion", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update], :duration => 6
    end
  end

	# en proyectos: elimina una fuente de verificación para un resultado
  def eliminar_fuente_verificacion_resultado
    @objeto = FuenteVerificacion.find(params[:id])
    @objeto.destroy
    @fuentes_verificacion = Resultado.find(params[:resultado_id]).fuente_verificacion.sort! {|x, y|  x.codigo <=> y.codigo }
    render :update do |page|
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "fuentes_verificacion", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

  #--
  # HIPOTESIS
  #++
  # en proyectos: prepara el formulario de edición o creación de una hipotesis para un objeto específico
  def nuevo_editar_hipotesis_objetivo_especifico
    @hipotesis = params[:id] ? Hipotesis.find(params[:id]) : Hipotesis.new
    render :partial => "formulario_hipotesis", :locals => { :objeto_id => params[:objetivo_especifico_id], :update => params[:update] }
  end

	# en proyectos: prepara el formulario de edición o creación de una hipotesis para un resultado
  def nuevo_editar_hipotesis_resultado
    @hipotesis = params[:id] ? Hipotesis.find(params[:id]) : Hipotesis.new
    render :partial => "formulario_hipotesis", :locals => { :objeto_id => params[:resultado_id], :update => params[:update]}
  end

	# en proyectos: modifica o crea una hipotesis para un objeto específico
  def crear_modificar_hipotesis_objetivo_especifico
    @hipotesis_actual = params[:id] ? Hipotesis.find(params[:id]) : Hipotesis.new
    @hipotesis_actual.update_attributes params[:hipotesis]
    ObjetivoEspecifico.find(params[:objetivo_especifico_id]).hipotesis  << @hipotesis_actual if @hipotesis_actual.errors.empty?
    @hipotesis = Hipotesis.find(:all, :conditions => { :objetivo_especifico_id => params[:objetivo_especifico_id]} )
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@hipotesis_actual) %>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "hipotesis", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

	# en proyectos: modifica o crea una hipotesis para un resultado
  def crear_modificar_hipotesis_resultado
    @hipotesis_actual = params[:id] ? Hipotesis.find(params[:id]) : Hipotesis.new
    @hipotesis_actual.update_attributes params[:hipotesis]
    Resultado.find(params[:resultado_id]).hipotesis  << @hipotesis_actual if @hipotesis_actual.errors.empty?
    @hipotesis = Hipotesis.find(:all, :conditions => { :resultado_id => params[:resultado_id]} )
    render :update do |page|
      page.replace_html 'formulario', :inline => '<%= mensaje_error(@hipotesis_actual) %>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "hipotesis", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

	# en proyectos: elimina una hipotesis para un objeto específico
  def eliminar_hipotesis_objetivo_especifico
    @objeto = Hipotesis.find(params[:id])
    @objeto.destroy
    @hipotesis = Hipotesis.find( :all, :conditions => { :objetivo_especifico_id => params[:objetivo_especifico_id] } )
    render :update do |page|
      page.replace_html params[:update], :partial => "hipotesis", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

	# en proyectos: elimina una hipotesis para un resultado
  def eliminar_hipotesis_resultado
    @objeto = Hipotesis.find(params[:id])
    @objeto.destroy
    @hipotesis = Hipotesis.find( :all, :conditions => { :resultado_id => params[:resultado_id] } )
    render :update do |page|
      page.replace_html params[:update], :partial => "hipotesis", :locals => { :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

  #--
  # SUBACTIVIDADES
  #++

  def subactividades
    @actividades = @proyecto.actividad
  end

  def editar_nueva_subactividad
    @subactividad = params[:id] ? Subactividad.find(params[:id]) : Subactividad.new
    render :partial => "subactividad"
  end

  def crear_modificar_subactividad
    subactividad = params[:id] ? Subactividad.find(params[:id]) : Subactividad.new
    subactividad.update_attributes params[:subactividad]
    subactividad.update_attribute("actividad_id", params[:actividad_id]) if subactividad.errors.empty?
    flash[:error] = subactividad
    redirect_to :action => "subactividades"
  end

  def eliminar_subactividad
    Subactividad.find(params[:id]).destroy
    redirect_to :action => "subactividades"
  end


  #--
  # LISTADO FUENTES DE VERIFICACION
  #++

  def listado_fuentes_verificacion
    @fuentes_verificacion = []
    @proyecto.objetivo_especifico.each  do |oe|
      @fuentes_verificacion += oe.fuente_verificacion
    end
    @proyecto.resultado.each  do |r|
      @fuentes_verificacion += r.fuente_verificacion
    end
  end

  def fuente_verificacion_editar_completada
    @fuente_verificacion = FuenteVerificacion.find(params[:id])
    render :partial => "formulario_fuente_verificacion_completada"
  end

  def fuente_verificacion_completada
    fuente_verificacion = FuenteVerificacion.find(params[:id])
    fuente_verificacion.update_attribute :completada, params[:fuente_verificacion][:completada]
    render :update do |page|
      page.call("Modalbox.hide")
      page.replace params[:update], :partial => "fuente_verificacion", :locals => { :fuente_verificacion => fuente_verificacion, :i =>  params[:i] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

  def fuente_verificacion_vincular_o_nuevo
    render :partial => "formulario_fuente_verificacion_vincular_o_nuevo"
  end

  def fuente_verificacion_editar_vincular_documento
    @documentos = @proyecto.documento.collect {|d| [d.adjunto_file_name, d.id]}
    render :partial => "formulario_fuente_verificacion_vincular"
  end


  def fuente_verificacion_vincular_documento
    FuenteVerificacionXDocumento.create :documento_id => params[:documento][:id], :fuente_verificacion_id => params[:fuente_verificacion_id]
    redirect_to :action => "listado_fuentes_verificacion"
  end

  def fuente_verificacion_eliminar_documento
    FuenteVerificacionXDocumento.find(:first, :conditions => {:documento_id => params[:id], :fuente_verificacion_id => params[:fuente_verificacion_id]}).destroy
    redirect_to :action => "listado_fuentes_verificacion"
  end


end


