#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de la entidad presupuesto. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para la gestión del presupuesto
#
class PresupuestoProyectosController < ApplicationController
  before_filter :filtrado_listados_inicial, :only => [ :listado, :listado_por_actividades]
  before_filter :verificar_etapa, :except => [:calcula_importe]
  before_filter :filtro_presupuesto_x_actividad, :only => [ :presupuesto_x_actividad, :modificar_crear_presupuesto_x_actividad, :eliminar_presupuesto_x_actividad ]
  before_filter :verificar_estado_proyecto, :only => [ :index, :modificar_crear, :eliminar, :modificar_crear_presupuesto_x_actividad, :eliminar_presupuesto_x_actividad ]

  def verificar_etapa
    if @proyecto.etapa.empty?
      flash[:error] = _("Tiene que definir por lo menos una etapa para acceder a la gestión del presupuesto")
      redirect_to :controller => :datos_proyecto, :action => :etapas
    end
  end

  def verificar_estado_proyecto
    unless @proyecto.estado_actual.definicion_estado.formulacion
      flash[:error] = _("El proyecto se encuentra en estado de ") + @proyecto.estado_actual.definicion_estado.nombre + _(". En este estado no se puede modificar el presupuesto. No ha sido definido como 'estado de formulacion' por su administrador.")
      redirect_to :action => :listado
    end
  end

  def filtrado_listados_inicial
    session[:presupuesto_proyectos_asc_desc] ||= "ASC"
    session[:presupuesto_proyectos_cadena_orden] ||= "partida.codigo"
    session[:presupuesto_proyectos_filtro_etapa] ||= "todas" #@proyecto.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.id
    session[:presupuesto_proyectos_filtro_agente] ||= "todos"
    session[:presupuesto_proyectos_filtro_moneda] ||= "todas"
    session[:presupuesto_proyectos_filtro_partida] ||= "todas"
  end

  # --
  # METODOS DE ENTRADA A PRESUPUESTO. Seleccion de ETAPA
  # ++

  # en proyectos: se redirecciona por defecto a listado

  def index
    redirect_to :action => :listado
  end

  # en proyectos: establece los parametros de ordenación
  def ordenado
    session[:presupuesto_proyectos_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:presupuesto_proyectos_cadena_orden] = session[:presupuesto_proyectos_orden] = params[:orden] ? params[:orden] : "fecha_inicio"
    session[:presupuesto_proyectos_cadena_orden] = "partida.codigo" if session[:presupuesto_proyectos_orden] =~ /^partida\.codigo_nombre/
    redirect_to :action => "listado"
  end

  def filtrado
    session[:presupuesto_proyectos_filtro_etapa] = params[:filtro][:etapa]
    session[:presupuesto_proyectos_filtro_agente] = params[:filtro][:agente]
    session[:presupuesto_proyectos_filtro_moneda] = params[:filtro][:moneda]
    session[:presupuesto_proyectos_filtro_partida] = params[:filtro][:partida]
    redirect_to :action => params[:listado]
  end

  def elementos_filtrado
    filtro_etapa = [["Todas", "todas"]] + @proyecto.etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    filtro_agente = [["Todos", "todos"]]  + @proyecto.implementador.collect{ |e| [e.nombre, e.id] }
    filtro_moneda = [["Todas", "todas"]]  + @proyecto.moneda.collect{ |e| [e.nombre, e.id] }
    filtro_partida = [["Todas", "todas"]] + Partida.find(:all).collect {|p| [p.codigo_nombre(@proyecto.id), p.id]}
    @opciones_filtrado = [{:rotulo =>  _("Selecciona etapa: "), :nombre => "etapa", :opciones => filtro_etapa},
                          {:rotulo =>  _("Selecciona moneda: "), :nombre => "moneda", :opciones => filtro_moneda},
                          {:rotulo =>  _("Selecciona agente: "), :nombre => "agente", :opciones => filtro_agente},
			  {:rotulo =>  _("Selecciona partida: "), :nombre => "partida", :opciones => filtro_partida} ]

    @etapa = Etapa.find( session[:presupuesto_proyectos_filtro_etapa]) unless session[:presupuesto_proyectos_filtro_etapa] == "todas"
    @estado_filtrado = [(session[:presupuesto_proyectos_filtro_etapa] == "todas" ? "Cualquier etapa" : (@etapa.nombre + " (" + @etapa.fecha_inicio.to_s + "/" + @etapa.fecha_fin.to_s + ")")), 
                       (session[:presupuesto_proyectos_filtro_moneda] == "todas" ? "Cualquier moneda" : Moneda.find(session[:presupuesto_proyectos_filtro_moneda]).nombre), 
                       (session[:presupuesto_proyectos_filtro_agente] == "todos" ? "Cualquier agente" : Agente.find(session[:presupuesto_proyectos_filtro_agente]).nombre),
		       (session[:presupuesto_proyectos_filtro_partida] == "todas" ? "Cualquier partida" : Partida.find(session[:presupuesto_proyectos_filtro_partida]).nombre) ] 
    @resumen = {:url => {:action => :presupuesto, :controller => :resumen_proyecto, :sin_layout => true}}
  end

  # --
  # METODOS DE GESTION DE Presupuesto
  # ++
  
  # en proyectos: listado de presupuestos del proyecto cargado en la sessión
  def listado
    condiciones = { "proyecto_id" => @proyecto.id}
    #condiciones["presupuesto.fecha_inicio"] = @etapa.fecha_inicio..@etapa.fecha_fin
    #condiciones["presupuesto.fecha_fin"] = @etapa.fecha_inicio..@etapa.fecha_fin
    condiciones["presupuesto.etapa_id"] = session[:presupuesto_proyectos_filtro_etapa] unless session[:presupuesto_proyectos_filtro_etapa] == "todas"
    condiciones["presupuesto.agente_id"] = session[:presupuesto_proyectos_filtro_agente] unless session[:presupuesto_proyectos_filtro_agente] == "todos"
    condiciones["presupuesto.moneda_id"] = session[:presupuesto_proyectos_filtro_moneda] unless session[:presupuesto_proyectos_filtro_moneda] == "todas"
    condiciones["presupuesto.partida_id"] = session[:presupuesto_proyectos_filtro_partida] unless session[:presupuesto_proyectos_filtro_partida] == "todas"
    @presupuestos =  Presupuesto.paginate( :all, :order => (session[:presupuesto_proyectos_cadena_orden] + " " + session[:presupuesto_proyectos_asc_desc]), 
                                           :include => [:partida, :libro, :subpartida],
                                           :conditions => condiciones, 
                                           :page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:presupuesto_numero]) )
    elementos_filtrado

    @formato_xls = @presupuestos.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "presupuesto"
        @objetos = @presupuestos
        render 'comunes_xls/listado', :layout => false
      end
    end
  end


  def datos_formulario
    @etapas = @proyecto.etapa.collect {|a|[a.nombre, a.id]}
    @partidas = Partida.all(:order => "codigo").collect {|a| [a.codigo_nombre(@proyecto.id), a.id]}
    @implementadores = @proyecto.implementador.all(:order => "nombre").collect {|a|[a.nombre, a.id]}
    @monedas = @proyecto.moneda.all(:order => "abreviatura").collect {|a| [a.abreviatura, a.id]} 
  end

  def nuevo_copiar_datos
    presupuesto_copiar = Presupuesto.find(params[:id])
    @presupuesto = @objeto = Presupuesto.new
    Presupuesto.datos_basicos_igualar(@presupuesto, presupuesto_copiar)  
    @actividades = presupuesto_copiar.presupuesto_x_actividad
    @financiadores = presupuesto_copiar.presupuesto_x_agente
    datos_formulario
    render :partial => "formulario"
  end

  # en proyectos: prepara el formulario de edición o creación de presupuesto
  def editar_nuevo
    @presupuesto = @objeto = params[:id] ?  Presupuesto.find(params[:id]) : Presupuesto.new
    @actividades = @presupuesto.presupuesto_x_actividad
    @financiadores = @presupuesto.presupuesto_x_agente
    datos_formulario
    render :partial => "formulario"
  end

  # en proyectos: modifica o crea un presupuesto
  def modificar_crear
    session[:presupuesto_proyectos_ultimo] = params
    #etapa = Etapa.find( session[:presupuesto_proyectos_filtro_etapa] )
    @presupuesto = params[:id] ?  Presupuesto.find(params[:id]) : Presupuesto.new()
    @presupuesto.proyecto_id = @proyecto.id
    #Recogemos la etapa anterior para verificar si hay que cambiar el detalle
    etapa_anterior_id =  @presupuesto.etapa_id 
    # Asignamos los valores al presupueseto sin guadarlos para poder comprobar_fechas_etapa y comprobar_concepto_unico
    @presupuesto.attributes = params[:presupuesto]
    @presupuesto.save if @presupuesto.errors.empty?
    if @presupuesto.errors.empty?
      @presupuesto.subpartida.update_attribute(:proyecto_id, @proyecto.id)  if @presupuesto.subpartida
      # Dividimos el presupuesto entre toda la etapa si es un nuevo presupuesto o la etapa ha variado
      @presupuesto.dividir_por_mes(@presupuesto.etapa.fecha_inicio, @presupuesto.etapa.fecha_fin) if (params[:id].nil? or etapa_anterior_id != @presupuesto.etapa_id)
      # Si se manda sin detalle tenemos que especificar e igualar los importes al del presupuesto
      if params["actividades_financiadores"]["detallado"] == "no"
        params["actividades"]["0"]["importe_convertido"], params["financiadores"]["0"]["importe_convertido"] = @presupuesto.importe, @presupuesto.importe 
        params["actividades"]["0"]["numero_unidades"] = @presupuesto.numero_unidades
      end
      @presupuesto.actualizar_presupuesto_x_actividad params[:actividades] if params["dividir_actividades"]["todas"] == "0"
      @presupuesto.dividir_por_actividades @proyecto.actividad if params["dividir_actividades"]["todas"] == "1"
      @presupuesto.actualizar_presupuesto_x_agente params[:financiadores]
    end     
    flash[:objeto_modificado] = {:errors => @presupuesto.errors , :id => @presupuesto.id } if @presupuesto.id
    flash[:objeto_modificado] = {:errors => @presupuesto.errors, :recuperar => true } unless @presupuesto.id
    @objeto_modificado = @presupuesto  
    redirect_to :action => "listado"
  end
 
  def actividades_financiadores
    @presupuesto = @objeto = params[:id] ?  Presupuesto.find(params[:id]) : Presupuesto.new
    @actividades = @presupuesto.presupuesto_x_actividad
    @financiadores = @presupuesto.presupuesto_x_agente
    render :update do |page|
      page.replace "actividades_financiadores", :partial => "comunes/actividades_financiadores"
      page.call("Modalbox.resizeToContent");
    end 
  end


  # en proyectos: añade un financiador al formulario
  def anadir_financiador
    render :template => "comunes/anadir_financiador"
  end

  # en proyectos: añade una actividad al formulario+
  def anadir_actividad
    render :template => "comunes/anadir_actividad"
  end

  # en proyectos: elimina un presupuesto
  def eliminar
    @objeto = Presupuesto.find(params[:id])
    @objeto.destroy
    #flash[:error] = @objetoinicio
    flash[:mensaje] = _("Se ha eliminado el presuesto.")
    redirect_to :action => 'listado'
  end
  
  
  def auto_complete_for_presupuesto_subpartida_nombre
    @subpartidas = Subpartida.find(:all, :conditions => ['nombre like ? and proyecto_id = ?', "%#{params[:search]}%", params[:proyecto_id].to_s])
    render :partial => "presupuesto_proyectos/auto_complete_for_presupuesto_subpartida_nombre"
  end

  # En los formularios de edicion de presupuesto, calcula precios segun lo que haya metido
  def calcula_importe
    update = params[:update]
    importe = moneda_a_float params[:presupuesto][:importe_convertido]
    numero_unidades = moneda_a_float params[:presupuesto][:numero_unidades] 
    coste_unitario = moneda_a_float params[:presupuesto][:coste_unitario_convertido] 
    # La preferencia de uso es importe, numero_unidades, coste_unitario
    if importe != 0 && numero_unidades != 0
      coste_unitario = importe / numero_unidades
    elsif importe != 0 && coste_unitario != 0
      numero_unidades = importe / coste_unitario
    elsif numero_unidades != 0 && coste_unitario != 0
      importe = numero_unidades * coste_unitario
    end
    render :update do |page|
      page[:presupuesto_coste_unitario_convertido].value = ('%.2f' % coste_unitario).to_s.sub(".",",")
      page[(update + "_numero_unidades").to_sym].value = numero_unidades
      page[(update + "_importe_convertido").to_sym].value = ('%.2f' % importe).to_s.sub(".",",")
    end
  end

  #-- 
  # METODOS DE PRESUPUESTO POR ACTIVIDAD 
  # ++

  # en proyectos: prepara el formulario de edición o creación de presupuesto
  def editar_nuevo_presupuesto_x_actividad
    @pxa = @objeto = params[:id] ?  PresupuestoXActividad.find(params[:id]) : PresupuestoXActividad.new
    @presupuesto = @pxa.presupuesto ? @pxa.presupuesto : Presupuesto.new
    datos_formulario
    render :partial => "formulario_presupuesto_x_actividad"
  end


  def listado_por_actividades
    @actividades = @proyecto.actividad
    elementos_filtrado
  end

  def presupuesto_x_actividad
    @actividad = Actividad.find_by_id(params[:id])
    render :update do |page|
      page.replace_html params[:update], :partial => "sublistado_actividades"
    end
  end

  def modificar_crear_presupuesto_x_actividad
    @actividad = Actividad.find_by_id(params[:actividad_id])
    @pxa = params[:id] ?  PresupuestoXActividad.find(params[:id]) : PresupuestoXActividad.new
    @presupuesto = @proyecto.presupuesto.find_by_concepto(params[:presupuesto][:concepto]) || Presupuesto.new
    ActiveRecord::Base.transaction do
      @presupuesto.presupuesto_x_actividad << @pxa
      @pxa.update_attributes params[:pxa]
      raise ActiveRecord::Rollback unless @pxa.errors.empty?
      @presupuesto.presupuesto_x_actividad << @pxa
      @presupuesto.importe = @presupuesto.presupuesto_x_actividad.inject(0) {|suma, pxa| suma + pxa.importe}
      @presupuesto.numero_unidades = @presupuesto.presupuesto_x_actividad.inject(0) {|suma, pxa| suma + pxa.numero_unidades}
      @presupuesto.attributes = params[:presupuesto]
      @presupuesto.proyecto_id = @proyecto.id
      @presupuesto.save
      @presupuesto.subpartida.update_attribute :proyecto_id, @proyecto.id if @presupuesto.subpartida
      raise ActiveRecord::Rollback unless @pxa.errors.empty? and @presupuesto.errors.empty?
    end
    #@pxa.destroy unless @presupuesto.errors.empty?
    render :update do |page|
      page.call("Modalbox.hide")
      page.replace_html params[:update], :partial => "sublistado_actividades"
      page.visual_effect :highlight, params[:update] , :duration => 6
      @objeto_modificado = {:errors =>  (@presupuesto.errors.empty? ? @pxa.errors : @presupuesto.errors) }
      @objeto_modificado[:url] =  {:action => :editar_nuevo_presupuesto_x_actividad, :id => @pxa.id, :actividad_id => params[:actividad_id], :update => params[:update] } if PresupuestoXActividad.exists?(@pxa.id)
      page.replace_html 'mensajes_modificacion', :inline => '<%= mensaje_objeto_modificado(@objeto_modificado) %>'
    end
  end



  def eliminar_presupuesto_x_actividad
    @actividad = Actividad.find_by_id(params[:actividad_id])
    @pxa = PresupuestoXActividad.find(params[:id])
    presupuesto = @pxa.presupuesto
    @pxa.destroy
    # recalculamos unidades e importe
    presupuesto.importe = presupuesto.presupuesto_x_actividad.inject(0) {|suma, pxa| suma + pxa.importe}
    presupuesto.numero_unidades = presupuesto.presupuesto_x_actividad.inject(0) {|suma, pxa| suma + pxa.numero_unidades}
    presupuesto.save unless presupuesto.importe == 0
    presupuesto.destroy if presupuesto.importe == 0
    render :update do |page|
      page.replace_html params[:update], :partial => "sublistado_actividades"
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@pxa, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

  def auto_complete_for_presupuesto_concepto
    @presupuesto = Presupuesto.find(:all, :conditions => ['presupuesto.concepto like ? and proyecto_id = ?', "%#{params[:search]}%", @proyecto.id])
    render :partial => "presupuesto_proyectos/auto_complete_for_presupuesto_concepto"
    #render :update do |page|
    #  page.auto_complete_result(@presupuesto, concepto) # "presupuesto_concepto_auto_complete", :inline => '<%= auto_complete_result @presupuesto, :concepto %>'
    #  page.show  "presupuesto_concepto_auto_complete"
    #  page.call("Modalbox.resizeToContent")
    #end 
  end

  #-- 
  # METODOS DE RECUPERAR EL ULTIMO PRESUPUESTO EDITADO
  # ++

  def editar_ultimo
    @presupuesto = @objeto = Presupuesto.new(session[:presupuesto_proyectos_ultimo]["presupuesto"])
    @actividades = [PresupuestoXActividad.new(session[:presupuesto_proyectos_ultimo]["actividades"]["0"])]
    @financiadores = [PresupuestoXAgente.new(session[:presupuesto_proyectos_ultimo]["financiadores"]["0"])]
    datos_formulario
    render :partial => "formulario"
  end

  private
    def float_a_moneda cadena
      ('%.2f' % numero).to_s.sub(".",",") if numero
    end

    def moneda_a_float cadena
      if cadena
        # primero comprobamos que el punto corresponde a miles ( formato => n.nnn )
        # y solo en ese caso lo eliminamos => Permitimos especificar decimales con punto
        numero = (cadena =~ /[\d]+\.[\d]{3}/) ? cadena.delete(".") : cadena
        # Si se esta usando la coma decimal, la cambiamos por un punto ( formato => n,nn o n,n )
        # REVISAR: Esto permite que haya coma para miles, pero impide que metan mas de 2 decimales con coma
        numero = numero.sub(/,/,".") if numero =~ /[\d]+,[\d]{1,2}$/
        return numero.to_f
      end
    end

    def filtro_presupuesto_x_actividad
      @etapa = Etapa.find( session[:presupuesto_proyectos_filtro_etapa] ) unless session[:presupuesto_proyectos_filtro_etapa] == "todas"
      @condiciones = { "presupuesto.proyecto_id" => @proyecto.id }
      @condiciones["presupuesto.etapa_id"] = @etapa.id unless session[:presupuesto_proyectos_filtro_etapa] == "todas"
      @condiciones["presupuesto.agente_id"] = session[:presupuesto_proyectos_filtro_agente] unless session[:presupuesto_proyectos_filtro_agente] == "todos"
      @condiciones["presupuesto.moneda_id"] = session[:presupuesto_proyectos_filtro_moneda] unless session[:presupuesto_proyectos_filtro_moneda] == "todas"
      @condiciones["presupuesto.partida_id"] = session[:presupuesto_proyectos_filtro_partida] unless session[:presupuesto_proyectos_filtro_partida] == "todas"
    end
end
