#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de los resumenes de proyecto. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para presentar un resumen de proyecto según diversos parámetros

class ResumenProyectoController < ApplicationController

  before_filter :criterios_gasto, :only => [:gasto]
  before_filter :criterios_presupuesto, :only => [:presupuesto]

  def criterios_gasto
    if params[:financiador_implementador] == "" 
      flash[:error] = _("Debe seleccionar agente. Para ello primero seleccione tipo implementador o financiador.")
      redirect_to :action => "gasto", :sin_layout =>  (params[:sin_layout] ? true : false)
    end
  end

  def criterios_presupuesto
    if params[:selector] and (params[:selector][:etapa].nil? or params[:selector][:etapa]=="" )
      flash[:error] = _("Debe seleccionar etapa. Si no existe ninguna debe dar de alta para este proyecto.")
      redirect_to :action => "presupuesto", :sin_layout =>  (params[:sin_layout] ? true : false)
    end
  end



  def index
    redirect_to :action => 'presupuesto'
  end
  
        # en proyectos: selecciona la moneda y llama a estado_presupuesto si ya hay moneda y actividad/partida
  def presupuesto
    @listado_moneda = @proyecto.moneda.collect{|e| [e.nombre, e.id]} + [["Todas (con tasa aplicada)", "todas"]]
    @listado_etapa = @proyecto.etapa.collect{|e| [e.nombre, e.id]}

    # Hacemos la busqueda si se seleccionan los criterios
    if params[:selector] and params[:selector][:moneda] != '' and params[:selector][:partida_actividad] != '' and params[:selector][:etapa] != ''
      estado_presupuesto 
      respond_to do |format|
        format.html do 
          render :action => 'presupuesto' , :layout => (params[:sin_layout] ? false : true)
        end if params[:selector][:fichero] == "0"
        format.xls do
          @nombre = "Resumen Presupuesto"
          render 'comunes_xls/resumen_tabla', :layout => false
        end if params[:selector][:fichero] == "1"
      end
    else
      render :action => 'presupuesto' , :layout => (params[:sin_layout] ? false : true)
    end
  end

  def estado_presupuesto
    partidas = Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    actividades = @proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    implementadores = @proyecto.implementador.collect{|i| {"id" => i.id, "nombre" => i.nombre}}
    financiadores = @proyecto.financiador.collect{|i| {"id" => i.id, "nombre" => i.nombre}}
    partidas_proyecto = @proyecto.partida_financiacion.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
    
    case  [params[:selector][:partida_actividad],  params[:selector][:financiador_implementador]]
      when ["partida", "implementador"] then
         datos = VPresupuesto.partida_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = partidas, implementadores
      when ["partida", "financiador"] then
         datos = VPresupuesto.partida_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = partidas, financiadores
      when ["actividad", "implementador"] then
         datos = VPresupuesto.actividad_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]
         filas, columnas = actividades, implementadores
      when ["actividad", "financiador"] then
         datos = VPresupuesto.actividad_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = actividades, financiadores
      when ["partida_proyecto", "implementador"] then
         datos = VPresupuesto.partida_proyecto_implementador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = partidas_proyecto, implementadores
      when ["partida_proyecto", "financiador"] then
         datos = VPresupuesto.partida_proyecto_financiador @proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio]  
         filas, columnas = partidas_proyecto, financiadores
    end       

    titulo =[ "Resumen de presupuesto.",
               "Moneda: #{ params[:selector][:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:selector][:moneda]).abreviatura} " ,
               "Tasa cambio: #{ (params[:selector][:tasa_cambio] == '1' or params[:selector][:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
               "Etapa: #{ Etapa.find(params[:selector][:etapa]).nombre } ",
            	 "Filas: #{ params[:selector][:partida_actividad] }",
		           "Columnas: #{ params[:selector][:financiador_implementador]}" ]

    @tablas = [{:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => { :columna_suma => true, :fila_suma => true, :ancho_fila => ((@proyecto.financiador.count + 2) * 105 + 200), :fila_porcentaje => true , :columna_porcentaje => true} } ]
    @titulo, @titulo[0] = titulo[0..3], "Subpartidas por partida"

    # Buscamos las subpartidas para segunda tabla si estamos en un resumen por partidas o por partidas de proyecto.
    @subpartidas = Array.new
    if params[:selector][:partida_actividad] == "partida"
      for partida in Partida.all
          importe_partida = VPresupuesto.partida(@proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio], partida.id.to_s ).first[:importe]
          @subpartidas.push(:cabecera => [[ partida.codigo_nombre , "2" ], [importe_partida ,"1_2_td"]])
        for subpartida in partida.subpartida @proyecto.id
          importe = VPresupuesto.subpartida_partida(@proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio], partida.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : "sin subpartida"), importe]) if importe
        end
      end
    elsif  params[:selector][:partida_actividad] == "partida_proyecto"
      for partida_proyecto in @proyecto.partida_financiacion
          importe_partida = VPresupuesto.partida_proyecto(@proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio], partida_proyecto.id.to_s ).first[:importe]
          @subpartidas.push(:cabecera => [[ partida_proyecto.codigo_nombre , "2" ], [importe_partida ,"1_2_td"]])
        for subpartida in partida_proyecto.subpartida @proyecto.id
          importe = VPresupuesto.subpartida_partida_proyecto(@proyecto.id, params[:selector][:etapa], params[:selector][:moneda], params[:selector][:tasa_cambio], partida_proyecto.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : "Sin subpartida"), importe]) if importe
        end
      end
      
    end

  end

  # en proyectos: selecciona la moneda, los agentes
  # y llama a busqueda_presupuesto
  def gasto
    @listado_moneda = @proyecto.moneda.collect{|e| [e.nombre, e.id]} + [["Todas las monedas (con tasa aplicada)", "todas"]]
    @listado_agente = @proyecto.implementador.collect{|e| [e.nombre, e.id]} + [["Todos los agentes", "todos"]]
    # Hacemos la busqueda si se seleccionan los criterios
    unless params[:moneda].nil? or params[:partida_actividad].nil? or params[:financiador_implementador].nil? 
      estado_gasto
      respond_to do |format|
        format.html do 
          render :action => "gasto", :layout => (params[:sin_layout] ? false : true)
        end if params[:fichero] == '0'
        format.xls do
          @nombre = "Seguimiento Gasto"
          render 'comunes_xls/resumen_tabla', :layout => false
        end if params[:fichero] == "1"
      end
    else
      render :action => "gasto", :layout => (params[:sin_layout] ? false : true)
    end
  end

  def agentes
    render :partial => "agentes"
  end


  def estado_gasto
    @fecha_de_inicio = fecha params["fecha_inicio"]
    @fecha_de_fin = fecha params["fecha_fin"]
    filas = case params[:partida_actividad]
              when "partida" then Partida.find(:all, :order => "codigo").collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
              when "actividad" then @proyecto.actividad.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
              when "partida_proyecto" then @proyecto.partida_financiacion.collect{|i| {"id" => i.id, "nombre" => i.codigo_nombre}}
             end
    columnas = [{"id" => "1", "nombre" => "Presupuesto"}, {"id" => "2", "nombre" => "Gastos"}]

    case  [params[:partida_actividad],  params[:financiador_implementador]]
      when ["partida", "implementador"] then
         presupuestos = VPresupuestoDetallado.partida_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["partida", "financiador"] then
         presupuestos = VPresupuestoDetallado.partida_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["actividad", "implementador"] then
         presupuestos = VPresupuestoDetallado.actividad_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.actividad_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}


      when ["actividad", "financiador"] then
         presupuestos = VPresupuestoDetallado.actividad_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.actividad_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}


     when ["partida_proyecto", "implementador"] then      
         presupuestos = VPresupuestoDetallado.partida_proyecto_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_proyecto_de_implementador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

      when ["partida_proyecto", "financiador"] then
         presupuestos = VPresupuestoDetallado.partida_proyecto_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "1", "importe" => p.importe , "fila_id" => p.fila_id }}
         gastos = VGasto.partida_proyecto_de_financiador( @proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio]).collect {|p| {"columna_id" => "2", "importe" => p.importe , "fila_id" => p.fila_id }}

    end 
    datos = presupuestos + gastos 
    titulo =[ "Resumen de presupuesto.",
               "Moneda: #{ params[:moneda] == 'todas' ?  'Todas las monedas' : Moneda.find(params[:moneda]).abreviatura} " ,
               "Tasa cambio: #{ (params[:tasa_cambio] == '1' or params[:moneda] == 'todas') ? 'Aplicada': 'No aplicada'}" ,
               "Fechas: #{@fecha_de_inicio.to_s + '/' +  @fecha_de_fin.to_s} ",
            	 "Filas: #{ params[:partida_actividad] }",
		           "Agente: #{ params[:agente] == "todos" ? "Todos los agentes" : Agente.find(params[:agente]).nombre }",
               "Rol del agente: #{ params[:financiador_implementador]}"  ]
    @tablas = [{:titulo => titulo ,:filas => filas, :columnas => columnas, :datos => datos, :otros => {  :columna_resta => true, :fila_suma => true, :desviacion => true, :clases => ["","2","1_2_td"] } }]


    # Buscamos las subpartidas para segunda tabla si estamos en un resumen por partidas o por partidas de proyecto.
    @titulo, @titulo[0] = titulo[0..3], "Subpartidas por partida"
    @subpartidas = [{:cabecera => [ ["Partida/Subpartida","2"], ["Presupuesto", "1_2_td"], ["Gasto", "1_2_td"], ["Diferencia", "1_2_td"] ] } ]
    if params[:partida_actividad] == "partida"
      for partida in Partida.all
          importe_presupuesto = VPresupuestoDetallado.partida(@proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida.id.to_s ).first[:importe]
          importe_gasto = VGasto.partida(@proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida.id.to_s ).first[:importe]
          @subpartidas.push(:cabecera => [[ partida.codigo_nombre , "2" ], [importe_presupuesto ,"1_2_td"], [importe_gasto ,"1_2_td"],
                                          [((importe_presupuesto || 0)  - (importe_gasto || 0 ))  ,"1_2_td"]])
          for subpartida in partida.subpartida @proyecto.id
            importe_presupuesto = VPresupuestoDetallado.subpartida_partida(@proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
            importe_gasto = VGasto.subpartida_partida(@proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
            @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : "sin subpartida"), importe_presupuesto, importe_gasto, ((importe_presupuesto || 0)  - (importe_gasto || 0 ))])  if importe_presupuesto or importe_gasto
          end
      end
    elsif  params[:partida_actividad] == "partida_proyecto"
      for partida_proyecto in @proyecto.partida_financiacion
          importe_presupuesto = VPresupuestoDetallado.partida_proyecto(@proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida_proyecto.id.to_s ).first[:importe]
          importe_gasto = VGasto.partida_proyecto(@proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida_proyecto.id.to_s ).first[:importe]
          @subpartidas.push(:cabecera => [[ partida_proyecto.codigo_nombre , "2" ], [importe_presupuesto ,"1_2_td"], [importe_gasto ,"1_2_td"] , 
                                          [((importe_presupuesto || 0)  - (importe_gasto || 0 ))  ,"1_2_td"]])
        for subpartida in partida_proyecto.subpartida @proyecto.id
          importe_presupuesto = VPresupuestoDetallado.subpartida_partida_proyecto(@proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida_proyecto.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          importe_gasto = VGasto.subpartida_partida_proyecto(@proyecto.id, params[:moneda], params[:agente], @fecha_de_inicio, @fecha_de_fin , params[:tasa_cambio], params[:financiador_implementador], partida_proyecto.id.to_s , (subpartida ? subpartida.id.to_s : nil)).first[:importe]
          @subpartidas.push(:contenido => [(subpartida ? subpartida.nombre : "Sin subpartida"), importe_presupuesto, importe_gasto, ((importe_presupuesto || 0)  - (importe_gasto || 0 ))]) if importe_presupuesto or importe_gasto
        end
      end
    end


      
  

  end


	# en proyectos: selecciona la caja para sacar el informe de arqueo
  def arqueo_caja
    @listado_libros = eval( "@" + singularizar_seccion ).libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}
    @listado_etapa = [["Todas", "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    @listado_agentes = @proyecto.implementador.collect{|e| [e.nombre, e.id]}
    @listado_monedas = @proyecto.moneda.collect{|e| [e.nombre, e.id]}
    # Si hemos especificado un libro para ver el informe
    #if params[:selector] and params[:selector][:libro] and params[:selector][:libro] != ''
    if params[:selector] && (params[:selector][:tipo] == "agente" || params[:selector][:tipo] == "cuenta")
      condiciones = { }
      @libro = Libro.find_by_id(params[:selector][:libro]) if params[:selector][:tipo] == "cuenta"
      @agente = @proyecto.implementador.find_by_id(params[:selector][:agente]) if params[:selector][:tipo] == "agente"
      @moneda = Moneda.find_by_id(params[:selector][:moneda]) if params[:selector][:tipo] == "agente"
      @moneda = @libro.moneda if @libro 
      if params[:selector][:etapa] != 'todas'
        @etapa = Etapa.find_by_id(params[:selector][:etapa])
        condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin
      end
      # Condiciones comunes para gastos y movimientos
      condiciones[:moneda_id] = @moneda.id

      # Obtiene todos los gastos de la cuenta en el periodo
      gastos = @proyecto.gasto.find(:all, :conditions => condiciones)

      # Obtiene todas las transferencias de la cuenta o el agente en el periodo
      condiciones[:libro_id] = @libro.id if @libro
      condiciones["libro.agente_id"] = @agente.id if @agente

      transferencias = @proyecto.transferencia.all(:conditions => condiciones, :include => "libro")

      # Genera el array de gastos y transferencias para presentar
      @lineas = Array.new
      @filas_desglosado = Array.new
      total = Hash.new
      entrante=0
      saliente=0
      # Recorre y toma todos los datos de las transferencias
      transferencias.each do |transferencia|
        total[transferencia.tipo.capitalize] = Hash.new if total[transferencia.tipo.capitalize].nil?
        total[transferencia.tipo.capitalize][transferencia.entrante_saliente.capitalize] ||= 0
        total[transferencia.tipo.capitalize][transferencia.entrante_saliente.capitalize] += transferencia.importe
        entrante += transferencia.importe if transferencia.entrante_saliente=="entrante"
        saliente += transferencia.importe if transferencia.entrante_saliente!="entrante"
        @filas_desglosado.push( :contenido => [	transferencia.fecha, transferencia.tipo.capitalize, transferencia.observaciones ? transferencia.observaciones : '',
						transferencia.entrante_saliente=="entrante" ? transferencia.importe : '',
						transferencia.entrante_saliente!="entrante" ? transferencia.importe : '' ])
      end
      
      # Recorre y toma todos los datos de los pagos
      gastos.each do |gasto|
        gasto.pago.each do |pago|
          if (@agente && pago.libro && pago.libro.agente_id == @agente.id) || (@libro && pago.libro_id == @libro.id)
            total["Pago"] = Hash.new if total["Pago"].nil?
            total["Pago"]["Saliente"] = 0 if total["Pago"]["Saliente"].nil?
            total["Pago"]["Saliente"] += pago.importe
            saliente += pago.importe
            @filas_desglosado.push( :contenido => [	pago.fecha, _("Pago"), (pago.observaciones||gasto.concepto),
							'', pago.importe ] )
          end
        end
      end
      # Ordena por fecha
      @filas_desglosado.sort! { |x,y| x[:contenido][0] <=> y[:contenido][0] }

      # Prepara la salida de datos
      @lineas.push( :cabecera => [ [_("Fecha"),"1_2"], [_("Tipo"),"1_2"], [_("Concepto"),"3_2"], [_("Entradas"),"1_2_td"], [_("Salidas"),"1_2_td"] ])
      @lineas += @filas_desglosado
      @lineas.push( :contenido => [ '','','','','' ] )
      @lineas.push( :cabecera => [ [_("Totales"),"1_2"], ["","1_2"], ["","3_2"], [entrante,"1_2_td"], [saliente,"1_2_td"], [entrante - saliente,"1_2_td"] ])
      total.each do |k,v|
        @lineas.push( :contenido => [ '',k,'',v["Entrante"]||'',v["Saliente"]||'','' ] )
      end

      respond_to do |format|
        format.html do
          render :template => 'comunes/arqueo_caja', :layout => (params[:sin_layout] ? false : true)
        end if params[:selector][:fichero] == '0'
        format.xls do
          @nombre = "arqueo_caja"
          @titulo = _("Arqueo ") + (@libro ? _("Cuenta/Caja ") + @libro.nombre : _("Cuentas/Cajas de ") + @agente.nombre ) + " (" + @moneda.nombre + ") / Etapa: " +
			(@etapa ? @etapa.nombre + " (desde " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " hasta " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : "Todas")
          render 'comunes_xls/resumen_listado', :layout => false
        end if params[:selector][:fichero] == '1'
      end
    else
      render :template => "comunes/arqueo_caja", :layout => (params[:sin_layout] ? false : true)
    end
  end

	# en proyectos, vista resumen de transferencias (segun la hoja X.A.1 del informe AECID)
  def transferencia
    @listado_etapa = [["Todas", "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    if params[:selector] && params[:selector][:etapa]
      @etapa = params[:selector][:etapa] == 'todas' ? nil : @proyecto.etapa.find_by_id(params[:selector][:etapa])

      @lineas = Array.new

      # Estas van a ser condiciones generales
      moneda_principal = @proyecto.moneda_principal
      moneda_intermedia = @proyecto.moneda_intermedia
      condiciones = Hash.new
      condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin if @etapa
      condiciones[:proyecto_id] = @proyecto.id
      condiciones[:entrante_saliente] = "entrante"

      # Subvenciones
      condiciones[:moneda_id] = moneda_principal.id
      condiciones[:tipo] = "subvencion"
      @lineas.push( :cabecera => [["0", "1_3"], [_("Subvenciones Recibidas"), "4"] ] )
      @lineas.push( :cabecera => [	[_("Fecha"), "1_3"],    [moneda_principal.abreviatura + _(" recibidos"), "1_2_td"],
					[_("Gastos"), "1_2_td"], ["", "1_4"], [_("Financiador"), "1"], [_("Cuenta"), "1"]      ])
      transferencias_principal = Transferencia.all(:order => "fecha", :include => ["libro", "libro_segundo"], :conditions => condiciones)
      tot_recibidos = 0
      tot_gastos = 0 
      transferencias_principal.each do |t|
          gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }
          @lineas.push( :contenido => [t.fecha, t.importe, gasto_transferencia, "", t.agente.nombre, t.libro.nombre])
          tot_recibidos += t.importe
          tot_gastos += gasto_transferencia
      end
      @lineas.push( :contenido => [_("TOTALES"), tot_recibidos, tot_gastos] )
      @lineas.push( :contenido => [] )


      # Transferencias a la moneda principal por paises para el financiador principal
      condiciones.delete(:tipo)
      condiciones["libro_segundos_transferencia.moneda_id"] = moneda_principal.id
      condiciones[:agente_id] = @proyecto.agente_id
      @proyecto.pais.each do |p|
        @lineas.push( :cabecera => [["1A", "1_3"], [_("Transferencias ") + @proyecto.agente.nombre + _(" a cuentas ") + moneda_principal.abreviatura + " " + p.nombre, "4"] ] )
        @lineas.push( :cabecera => [	[_("Fecha"), "1_3"],	[moneda_principal.abreviatura + _(" transferidos"), "1_2_td"],
					[moneda_principal.abreviatura + _(" recibidos"), "1_2_td"], [_("Gastos"), "1_2_td"],
					["", "1_4"], [_("Cuenta Origen"), "1"],	[_("Cuenta Destino"), "1"]	])

        condiciones["libro.pais_id"] = p.id
        transferencias_principal = Transferencia.all(:order => "fecha", :include => ["libro", "libro_segundo"], :conditions => condiciones)
        tot_transferidos = 0
        tot_recibidos = 0
        tot_gastos = 0
        transferencias_principal.each do |t|
          gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }
          @lineas.push( :contenido => [t.fecha, t.transferencia_vinculada ? t.transferencia_vinculada.importe : "---", t.importe, gasto_transferencia, "", t.libro_receptor_emisor.nombre, t.libro.nombre])
          tot_transferidos += t.transferencia_vinculada.importe if t.transferencia_vinculada
          tot_recibidos += t.importe
	  tot_gastos += gasto_transferencia
        end
        @lineas.push( :contenido => [_("TOTALES"), tot_transferidos, tot_recibidos, tot_gastos] )
        @lineas.push( :contenido => [] )
      end

      # Transferencias a la divisa (si la hay) por paises y financiadores
      condiciones[:moneda_id] = moneda_intermedia.id if moneda_intermedia
      condiciones["libro_segundos_transferencia.moneda_id"] = moneda_principal.id
      @proyecto.financiador.each do |f|
        condiciones[:agente_id] = f.id
        @proyecto.pais.each do |p|
          condiciones["libro.pais_id"] = p.id
          transferencias_divisa = Transferencia.all(:order => "fecha", :include => ["libro", "libro_segundo"], :conditions => condiciones)

          @lineas.push( :cabecera => [["1B", "1_3"], [_("Transferencias ") + f.nombre + _(" a cuentas ") + moneda_intermedia.abreviatura + " " + p.nombre, "4"] ] )
          @lineas.push( :cabecera => [	[_("Fecha"), "1_3"],		[moneda_principal.abreviatura + _(" enviados"), "1_2_td"],
                                        [moneda_intermedia.abreviatura + _(" recibidos"), "1_2_td"],
					[_("Tipo de Cambio ") + moneda_intermedia.abreviatura + "/" + moneda_principal.abreviatura, "1_2_td"],
					[_("Gastos") + " (" + moneda_intermedia.abreviatura + ")", "1_2_td"],
                                        ["", "1_4"], [_("Cuenta Origen"), "1"], [_("Cuenta Destino"), "1"]     ])

          tot_transferidos = 0
          tot_recibidos = 0
          tot_gastos = 0
          transferencias_divisa.each do |t|
            gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }
            @lineas.push( :contenido => [t.fecha, t.transferencia_vinculada ? t.transferencia_vinculada.importe : "---", t.importe, t.tasa_cambio, gasto_transferencia, "", t.libro_receptor_emisor.nombre, t.libro.nombre])
            tot_transferidos += t.transferencia_vinculada.importe if t.transferencia_vinculada
            tot_recibidos += t.importe
            tot_gastos += gasto_transferencia
          end
          @lineas.push( :contenido => [_("TOTALES"), tot_transferidos, tot_recibidos, "", tot_gastos] )
          @lineas.push( :contenido => [] )
        end
      end if @proyecto.moneda_intermedia

      # Transferencias a las monedas locales por países
      condiciones.delete("libro_segundos_transferencia.moneda_id")
      condiciones.delete(:agente_id)
      @proyecto.pais.each do |p|
        condiciones["libro.pais_id"] = p.id
        p.moneda.each do |m|
          condiciones[:moneda_id] = m.id
          transferencias_locales = Transferencia.all(:order => "fecha", :include => ["libro"], :conditions => condiciones)

          @lineas.push( :cabecera => [["2", "1_3"], [p.nombre + " (" + m.nombre + ")", "4"] ] )
          cabecera  = [ [_("Fecha"), "1_3"], [moneda_principal.abreviatura + _(" recibidos y cambiados"), "1_2_td"]]
          cabecera += [ [moneda_intermedia.abreviatura + _(" recibidos y cambiados"), "1_2_td"] ] if moneda_intermedia
          cabecera += [ [m.abreviatura + _(" recibidos"), "1_2_td"], [_("Tasa de Cambio ") + m.abreviatura + "/" + moneda_principal.abreviatura, "1_2_td"] ]
          cabecera += [ [_("Tasa de Cambio ") + m.abreviatura + "/" + moneda_intermedia.abreviatura, "1_2_td"] ] if moneda_intermedia
          cabecera += [ [_("Gastos") + " (" + m.abreviatura + ")", "1_2_td"],
                        ["", "1_4"], [_("Financiador"), "3_4"], [_("Destinatario"), "3_4"], [_("Cuenta Origen"), "1"], [_("Cuenta Destino"), "1"]     ]
          @lineas.push( :cabecera => cabecera )

          tot_transferidos_princ = 0
          tot_transferidos_sec = 0
          tot_recibidos = 0
          tot_gastos = 0
          transferencias_locales.each do |t|
            if t.libro_segundo && (t.libro_segundo.moneda_id == moneda_principal.id || (moneda_intermedia && t.libro_segundo.moneda_id == moneda_intermedia.id))
              gasto_transferencia = t.gasto.inject(0) {|sum,g| sum+g.importe }

              if moneda_intermedia
                @lineas.push( :contenido => [t.fecha, t.transferencia_vinculada ? t.transferencia_vinculada.importe : "---", "", t.importe, t.tasa_cambio, "", gasto_transferencia, "", t.agente ? t.agente.nombre : "---", t.libro.agente.nombre, t.libro_receptor_emisor.nombre, t.libro.nombre]) if t.libro_segundo.moneda == moneda_principal
                @lineas.push( :contenido => [t.fecha, "", t.transferencia_vinculada ? t.transferencia_vinculada.importe : "---", t.importe, "", t.tasa_cambio, gasto_transferencia, "", t.agente ? t.agente.nombre : "---", t.libro.agente.nombre, t.libro_receptor_emisor.nombre, t.libro.nombre]) if t.libro_segundo.moneda == moneda_intermedia
              else
                @lineas.push( :contenido => [t.fecha, t.transferencia_vinculada ? t.transferencia_vinculada.importe : "---", t.importe, t.tasa_cambio, gasto_transferencia, "", t.agente ? t.agente.nombre : "---", t.libro.agente.nombre, t.libro_receptor_emisor.nombre, t.libro.nombre]) if t.libro_segundo.moneda == moneda_principal
              end

              tot_transferidos_princ += t.transferencia_vinculada.importe if t.transferencia_vinculada && t.libro_segundo.moneda == moneda_principal
              tot_transferidos_sec += t.transferencia_vinculada.importe if t.transferencia_vinculada && moneda_intermedia && t.libro_segundo.moneda == moneda_intermedia
              tot_recibidos += t.importe
              tot_gastos += gasto_transferencia
            end
          end
          @lineas.push( :contenido => [_("TOTALES"), tot_transferidos_princ, tot_transferidos_sec, tot_recibidos, "", "", tot_gastos] ) if moneda_intermedia
          @lineas.push( :contenido => [_("TOTALES"), tot_transferidos_princ, tot_recibidos, "", tot_gastos] ) unless moneda_intermedia
          @lineas.push( :contenido => [] )
        end
      end 

      @resumen_error = _("No existe ningún país definido para el proyecto.<br/>Configure los paises de actuación en las Relaciones del Menú de Configuración del proyecto.") if @proyecto.pais.size == 0

      respond_to do |format|
        format.html do
          render :layout => (params[:sin_layout] ? false : true)
        end if params[:selector][:fichero] == '0'
        format.xls do
          @nombre = "transferencias"
          @titulo = _("Resumen Transferencias ") + @proyecto.nombre + " / Etapa: " +
                        (@etapa ? @etapa.nombre + " (desde " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " hasta " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : "Todas")
          render 'comunes_xls/resumen_listado', :layout => false
        end if params[:selector][:fichero] == '1'
      end
    end
  end

	# En proyectos, vista resumen de estado de tesoreria (segun la hoja X.A.2 del informe AECID)
  def estado_tesoreria
    @listado_etapa = [["Todas", "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    @listado_agente = @proyecto.financiador.collect{|e| [e.nombre, e.id]}
    if params[:selector] && params[:selector][:agente]
      @etapa = params[:selector][:etapa] == 'todas' ? nil : @proyecto.etapa.find_by_id(params[:selector][:etapa])
      @agente = @proyecto.financiador.find_by_id(params[:selector][:agente])
      @lineas = Array.new
      monedas = @proyecto.moneda - [@proyecto.moneda_principal, @proyecto.moneda_intermedia]
      paises = @proyecto.pais

      # Transferencias de subvenciones (por agente y total) -> Habra problemas con financiaciones en otras monedas!!!
      principal_financia = @proyecto.transferido_total(@proyecto.moneda_principal, @etapa, @agente, "entrante", "subvencion", nil)
      total_financia = @proyecto.transferido_total(@proyecto.moneda_principal, @etapa, nil, "entrante", "subvencion", nil)

      # Dinero transferido a los paises (entrante y saliente) para moneda del proyecto e intermedia (evitando los paises que las tengan como local)
      principal_entrante = paises.collect do |p|
        @proyecto.transferido_total(@proyecto.moneda_principal, @etapa, @agente, "entrante", nil, p) unless p.moneda.find_by_id(@proyecto.moneda_id)
      end
      principal_saliente = paises.collect do |p|
        @proyecto.transferido_total(@proyecto.moneda_principal, @etapa, @agente, "saliente", nil, p) unless p.moneda.find_by_id(@proyecto.moneda_id)
      end
      divisa_saliente = []
      divisa_saliente = paises.collect do |p|
        @proyecto.transferido_total(@proyecto.moneda_intermedia, @etapa, @agente, "saliente", nil, p) unless p.moneda.find_by_id(@proyecto.moneda_intermedia_id)
      end if @proyecto.moneda_intermedia
      divisa_entrante = []
      divisa_entrante = paises.collect do |p|
        @proyecto.transferido_total(@proyecto.moneda_intermedia, @etapa, @agente, "entrante", nil, p) unless p.moneda.find_by_id(@proyecto.moneda_intermedia_id)
      end if @proyecto.moneda_intermedia
      principal_saliente.delete(nil)
      principal_entrante.delete(nil)
      divisa_saliente.delete(nil)
      divisa_entrante.delete(nil)
      divisa_total_entrante = principal_entrante + divisa_entrante
      divisa_total_saliente = principal_saliente + divisa_saliente

      # Moneda local recibida
      locales_entrante = paises.inject([]){ |sum, p| sum + p.moneda.collect{|m| @proyecto.transferido_total(m, @etapa, @agente, "entrante", nil, p)} }
      # Moneda local saliente
      locales_saliente = paises.inject([]){ |sum, p| sum + p.moneda.collect{|m| @proyecto.transferido_total(m, @etapa, @agente, "saliente", nil, p)} }

      # Transferido desde la moneda principal menos las trasferencias dentro de otros paises como divisa
      principal_transferido  = principal_saliente.inject(@proyecto.transferido_total(@proyecto.moneda_principal, @etapa, @agente, "saliente")){|sum,p| sum - p}
      #  o como moneda local igual a la principal
      paises.each do |p|
        principal_transferido -= @proyecto.transferido_total(@proyecto.moneda_principal, @etapa, @agente, "saliente", nil, p) if p.moneda.find_by_id(@proyecto.moneda_principal.id)
      end
      

      # Intereses a las cuentas en moneda principal en los paises
      intereses_principal_paises = paises.collect do |p|
        @proyecto.transferido_total(@proyecto.moneda_principal, @etapa, nil, "entrante", "intereses", p) unless p.moneda.find_by_id(@proyecto.moneda_id)
      end
      intereses_principal_paises.delete(nil)
      # Resto de intereses de la moneda principal
      intereses_principal = @proyecto.transferido_total(@proyecto.moneda_principal, @etapa, nil, "entrante", "intereses", nil)
      intereses_principal_paises.each{|i| intereses_principal -= i}
      # Quitando tambien las de las monedas locales igual a la principal (esto falta)
      paises.each do |p|
        intereses_principal -= @proyecto.transferido_total(@proyecto.moneda_principal, @etapa, nil, "entrante", "intereses", p) if p.moneda.find_by_id(@proyecto.moneda_principal.id)
      end


      # Intereses de la moneda principal
      intereses = [ intereses_principal ] + intereses_principal_paises
      # Intereses para divisa
      intereses += paises.collect do |p|
        @proyecto.transferido_total(@proyecto.moneda_intermedia, @etapa, nil, nil, "intereses", p) unless p.moneda.find_by_id(@proyecto.moneda_intermedia_id)
      end if @proyecto.moneda_intermedia
      intereses.delete(nil)
      # Intereses para monedas locales
      paises.each{|p| intereses += p.moneda.collect{|m| @proyecto.transferido_total(m, @etapa, nil, nil, "intereses", p) } }

      porcentaje_intereses = total_financia == 0 ? 0 : principal_financia / total_financia 
      intereses = intereses.collect { |v| v * porcentaje_intereses }

      # Tasas de Cambio
      # Hay que corregir la ultima fecha para que sea de TC y no de etapa 
      ultima_fecha = @etapa ? @etapa.fecha_fin : @proyecto.etapa.last(:order => "fecha_fin").fecha_fin

      #  tasa de cambio para moneda principal y divisa (si existe)
      tasa_cambio = [ 1 ] + principal_saliente.collect{|a| 1} 
      if @proyecto.moneda_intermedia_id
        gasto_falso_divisa = Gasto.new(:moneda_id => @proyecto.moneda_intermedia_id, :fecha => ultima_fecha)
        tc = TasaCambio.tasa_cambio_para_gasto(gasto_falso_divisa,@proyecto)
        tc_valor = tc ? tc.tasa_cambio : 0
        tasa_cambio += divisa_saliente.collect{|a| tc_valor }
      end 

      #  tasa de cambio por paises y monedas
      paises.each { |p| tasa_cambio += p.moneda.collect{|m| TasaCambio.tasa_cambio_para_gasto(Gasto.new(:moneda_id => m.id, :fecha => ultima_fecha), @proyecto).tasa_cambio} }

      # Gastos
      #  gastos en la moneda principal como divisa por paises
      gastos_principal = paises.collect do |p|
        @proyecto.gasto_total(@agente, @proyecto.moneda_principal, @etapa, p) unless p.moneda.find_by_id(@proyecto.moneda_id)
      end
      gastos_principal.delete(nil)
      
      #  y el resto de gastos de la moneda principal (el total menos en divisa)
      gastos_principal_local = gastos_principal.inject(@proyecto.gasto_total(@agente, @proyecto.moneda_principal, @etapa, nil)) { |sum,g| sum - g }
      #  quitando tambien los gastos de paises con moneda local como principal (esto falta)
      paises.each do |p|
        gastos_principal_local -= @proyecto.gasto_total(@agente, @proyecto.moneda_principal, @etapa, p) if p.moneda.find_by_id(@proyecto.moneda_principal.id)
      end

      # El conjunto total de gastos: Al principal local + principal como divisa 
      #   le incluimos las divisas
      gastos  = [gastos_principal_local] + gastos_principal 
      gastos += paises.collect do |p|
        @proyecto.gasto_total(@agente, @proyecto.moneda_intermedia, @etapa, p) unless p.moneda.find_by_id(@proyecto.moneda_intermedia_id)
      end if @proyecto.moneda_intermedia
      #   y las monedas de cada pais
      paises.each{|p| gastos += p.moneda.collect{|m| @proyecto.gasto_total(@agente, m, @etapa, p)} }
      gastos.delete(nil)

      # Totales
      total_fondos = [ principal_financia - principal_transferido] + 
			divisa_total_entrante.zip(divisa_total_saliente).map {|a| a.inject(:-)} +
			locales_entrante.zip(locales_saliente).map {|a| a.inject(:-)}
      total_fondos = total_fondos.zip(intereses).map {|a| a.inject(:+)}

      saldos = total_fondos.zip(gastos).map {|a| a.inject(:-)}
      saldos_convertidos = saldos.zip(tasa_cambio).map {|a| a.inject(:*)}
      total_euros = saldos_convertidos.inject(0){|sum, sc| sum + sc}

      # Construye la cabecera
      cabecera = [ [_("Ingresos y Transferencias"),"1"], [@proyecto.moneda_principal.nombre, "1_2_td"] ]
      # Recoge divisas en moneda principal
      cabecera += paises.collect do |p|
        [_("Divisa ") + @proyecto.moneda_principal.abreviatura + " " + p.nombre, "1_2_td"] unless p.moneda.find_by_id(@proyecto.moneda_id)
      end
      # Y la intermedia
      cabecera += paises.collect do |p|
        [_("Divisa ") + @proyecto.moneda_intermedia.abreviatura + " " + p.nombre, "1_2_td"] unless @proyecto.moneda_intermedia.nil? || p.moneda.find_by_id(@proyecto.moneda_intermedia_id)
      end
      # E incluye el resto de monedas para los demas paises
      paises.each { |p| cabecera += p.moneda.collect{|m| [m.nombre + " " + p.nombre, "1_2_td"]} }
      cabecera.delete(nil)

      @lineas.push( :cabecera => cabecera)
      @lineas.push( :contenido => [_("(+) Saldos de periodos anteriores")] )
      @lineas.push( :contenido => [_("(+) Subvención recibida ") + @proyecto.agente.nombre, principal_financia ] )
      @lineas.push( :contenido => [_("(-) Total ") + @proyecto.moneda_principal.nombre.pluralize + _(" transferidos "), principal_transferido ])
      @lineas.push( :contenido => [_("(+) Total Divisa intermedia"), ""] + divisa_total_entrante )
      @lineas.push( :contenido => [_("(-) Total Divisa recibida y cambiada"), ""] + divisa_total_saliente )
      @lineas.push( :contenido => [_("(+) Total Moneda Local obtenida"), ""] + divisa_total_saliente.collect{|a| ""} + locales_entrante )
      @lineas.push( :contenido => [_("(-) Moneda Local recibida y cambiada"), "" ] + divisa_total_saliente.collect{|a| ""} + locales_saliente )
      @lineas.push( :contenido => [_("(+) TOTAL INTERESES GENERADOS")] + intereses )
      @lineas.push( :contenido => [_("(I) TOTAL Fondos de la subvencion")] + total_fondos )
      @lineas.push( :contenido => [_("(II) GASTOS aplicados (-)")] + gastos )
      @lineas.push( :contenido => [_("(III) SALDOS CORRIENTES (I) - (II)")] + saldos )
      @lineas.push( :contenido => [_("Tipo de cambio aplicable")] + tasa_cambio )
      @lineas.push( :contenido => [_("(IV) SALDO ") + @proyecto.moneda_principal.nombre.pluralize + _(" EQUIVALENTES")] + saldos_convertidos )
      @lineas.push( :contenido => [_("(V) TOTAL ") + @proyecto.moneda_principal.nombre.pluralize, total_euros ] )

      respond_to do |format|
        format.html do
          render :layout => (params[:sin_layout] ? false : true)
        end if params[:selector][:fichero] == '0'
        format.xls do
          @nombre = "tesoreria"
          @titulo = _("Resumen Tesoreria ") + @proyecto.nombre + " / Etapa: " +
                        (@etapa ? @etapa.nombre + " (desde " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " hasta " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : "Todas")
          render 'comunes_xls/resumen_listado', :layout => false
        end if params[:selector][:fichero] == '1'
      end

    end
  end

	# En proyectos, vista resumen de la matriz por etapa y pais
  def matriz
    @listado_etapa = [["Todas", "todas"]] + @proyecto.etapa.collect{|e| [e.nombre, e.id]}
    @listado_pais =  [["Todos", "todos"]] + @proyecto.pais.collect{ |p| [p.nombre, p.id] }.sort{ |a, b| a[0] <=> b[0]}
    if params[:selector]
      @pais = Pais.find(params[:selector][:pais]) if params[:selector][:pais] != "todos"
      @etapa = Etapa.find(params[:selector][:etapa]) if params[:selector][:etapa] != "todas"
      @columnas_nombre = [_("descripcion"), _("indicadores"), _("fuentes de verificacion"), _("hipótesis")]
      @lineas = Array.new
      condiciones = { "proyecto_id" =>  @proyecto.id}
      condiciones["actividad_x_pais.pais_id"] = params[:selector][:pais] if params[:selector][:pais] != "todos"
      condiciones["actividad_x_etapa.etapa_id"] = params[:selector][:etapa] if params[:selector][:etapa] != "todas"
      filas_objetivos = Array.new
      filas_resultados = Array.new
      filas_actividades = Array.new 
      @proyecto.objetivo_especifico.all.each do |objetivo|
        existen_actividades = false
        objetivo.resultado.all.each do |resultado|
          existen_actividades = false
          actividades = resultado.actividad.find(:all, :include => [:actividad_x_pais, :actividad_x_etapa], :conditions => condiciones )
          actividades.each do |actividad|
            nuevas_actividades = desglosa_actividades(actividad,@etapa)
            filas_actividades += nuevas_actividades 
            existen_actividades = true if nuevas_actividades.length > 0 
          end
          filas_resultados += desglosa_fuentes(resultado) if existen_actividades
        end
        filas_objetivos += desglosa_fuentes(objetivo) if existen_actividades 
      end
      # Va rellenando las lineas
      @lineas.push(:cabecera => [ [_("Descripcion"),"3_2"], [_("Indicadores"),"1"], [_("Fuentes de verificacion"),"1"], [_("Hipótesis"),"1"] ]) 
      @lineas += filas_objetivos
      @lineas.push(:cabecera => [ [_("Resultados"),"3_2"], ["","1"], ["","1"], ["","1" ] ])
      @lineas += filas_resultados 
      @lineas.push(:cabecera => [ [_("Actividad"),"3_2"], [_("Recursos"),"1_td"], [_("Costes"),"1_td"] ])
      @lineas += filas_actividades

      respond_to do |format|
        format.html do
          render :action => "matriz", :layout => (params[:sin_layout] ? false : true)
        end if params[:selector][:fichero] == '0'
        format.xls do
          @nombre = "matriz"
          @titulo = "Matriz " + @proyecto.nombre + " / Pais: " + (@pais ? @pais.nombre : "Todos") + " / Etapa: " +
			(@etapa ? @etapa.nombre + " (desde " + @etapa.fecha_inicio.strftime('%d/%m/%Y') + " hasta " + @etapa.fecha_fin.strftime('%d/%m/%Y') + ")" : "Todas")
          render 'comunes_xls/resumen_listado', :layout => false
        end if params[:selector][:fichero] == '1'
      end
    else
      render :action => "matriz", :layout => (params[:sin_layout] ? false : true)
    end

  end

private


	# Para un objetivo especifico o un resultado, saca hipotesis, indicadores y fuentes de verificacion
  def desglosa_fuentes(objeto)
    filas = Array.new
    hipotesis = objeto.hipotesis
    indicadores = objeto.indicador
    objeto_repetido = nil
    # Si no hay indicadores al menos muestra el codigo del objeto
    filas.push( :contenido => [ objeto.codigo + ": " + objeto.descripcion, "", "", "" ] ) if indicadores.length == 0
    # Si hay indicadores se los recorre para rellenar las filas
    indicadores.each do |indicador|
      indicador_repetido = nil
      fuentes = indicador.fuente_verificacion
      # Si no hay fuentes de verificacion al menos pone el indicador
      filas.push( :contenido => [	objeto_repetido ? "" : objeto.codigo + ": " + objeto.descripcion,
					indicador.codigo + ": " + indicador.descripcion,
					"",
					(hipotesis.length>0 ? hipotesis.shift.descripcion : "") ]) if fuentes.length == 0
      # Si hay fuentes de verificacion las recorre para rellenar las filas
      fuentes.each do |fuente|
        filas.push( :contenido => [	objeto_repetido ? "" : objeto.codigo + ": " + objeto.descripcion,
					indicador_repetido ? "" : indicador.codigo + ": " + indicador.descripcion,
					fuente.codigo + ": " + fuente.descripcion,
					(hipotesis.length>0 ? hipotesis.shift.descripcion : "") ])
        objeto_repetido = true
        indicador_repetido = true
      end
      objeto_repetido = true
    end
    # Si queda alguna hipotesis la mete
    hipotesis.each do |hip|
      filas.push( :contenido => [	"", "", "", hip.descripcion ]) 
    end
    return filas
  end

	# Para una actividad, saca partidas de financiador e importes totales
  def desglosa_actividades(objeto,etapa)
    hash = Hash.new
    filas = Array.new
    importe_total = 0
    # Se recorre todos los presupuestos por actividad
    objeto.presupuesto_x_actividad.each do |pxa|
      # Y en el caso de que el presupuesto sea de la etapa relacionada y tenga asignada una partida lo mete
      if etapa.nil? || (pxa.presupuesto && pxa.presupuesto.etapa_id == etapa.id)
        partida_asociada=pxa.presupuesto.partida.partida_asociada(@proyecto.id) if pxa.presupuesto && pxa.presupuesto.partida
        if partida_asociada
          # aplica la tasa de cambio
          importe = pxa.importe * pxa.presupuesto.tasa_cambio
          hash[partida_asociada.nombre] += importe if hash[partida_asociada.nombre]
          hash[partida_asociada.nombre] = importe unless hash[partida_asociada.nombre]
          importe_total += importe
        end
      end
    end
    # Si no hay presupuesto para la actividad, mete al menos la actividad
    filas.push( :contenido => [		objeto.codigo + ": " + objeto.descripcion, "", ""]) if hash.length == 0
    # Si hay datos en el hash anterior, va rellenando las lineas
    if importe_total != 0
      filas.push( :contenido => [	objeto.codigo + ": " + objeto.descripcion, "", importe_total])
    end
    hash.each do |k,v|
      filas.push( :contenido => [	"", k, v ])
    end
    return filas
  end

end
#done

