#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de Transferencias. Este controlador es utilizado desde las secciones:
# * Sección agentes: se utiliza para gestionar las transferencias de un determinado agente
# * Sección proyectos: se utiliza para gestionar las transferencias de un proyecto

class TransferenciaController < ApplicationController
  

  before_filter :verificar_estado_proyecto, :only => [ :index, :modificar_crear, :eliminar, :guardar_vinculacion]

  def verificar_estado_proyecto
    unless @proyecto.estado_actual and @proyecto.estado_actual.definicion_estado.aprobado
      flash[:error] = _("El proyecto se encuentra 'sin estado'. En este estado no se pueden modificar los datos del proyecto.")    if @proyecto.estado_actual.nil?
      flash[:error] = _("E  l proyecto se encuentra en estado de ") + @proyecto.estado_actual.definicion_estado.nombre + _(". En este estado no se pueden modificar los movimientos. No ha sido definido como 'aprobado' por su administrador.") unless @proyecto.estado_actual.nil?
      redirect_to :action => "listado" unless params[:action] == "index"
      redirect_to :action => :filtrado_ordenado_iniciales if params[:action] == "index"
    end if params[:seccion] == "proyectos"
  end
  
  # en proyectos y en agente: se redirecciona por defecto a filtrado_ordenado_iniciales creando el mensaje en funcion de la sección.
  def index
    objeto = singularizar_seccion
    redirect_to :action => :filtrado_ordenado_iniciales
  end

  # en proyectos y en agente: inicializa los defectos para ordenado y filtro y redirecciona a listado
  def filtrado_ordenado_iniciales
    if eval( "@" + singularizar_seccion ).etapa.empty? 
      flash[:error] = _("Tiene que definir por lo menos una etapa para acceder a la gestión de transferencias.")
      redirect_to :controller => :datos_proyecto, :action => :etapas
    else
      session[:transferencia_asc_desc] ||= "ASC" 
      session[:transferencia_cadena_orden] ||= "fecha"
      session[:transferencia_filtro_etapa] ||= "todas"
      session[:transferencia_filtro_libro] ||= "todas"
      session[:transferencia_filtro_tipo] ||= "todos"
      session[:transferencia_filtro_entrante_saliente] ||= "todas"
      redirect_to :action => :listado
    end
  end

  # en proyectos y agentes: establece los parametros de ordenación
  def ordenado
    session[:transferencia_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:transferencia_cadena_orden] = params[:orden] ? params[:orden] : "fecha"
    redirect_to :action => "listado"
  end

  # en proyectos y agentes: establece los parametros de filtro
  def filtrado
    session[:transferencia_filtro_etapa] = params[:filtro][:etapa]
    session[:transferencia_filtro_tipo] = params[:filtro][:tipo]
    session[:transferencia_filtro_libro] = params[:filtro][:libro]
    session[:transferencia_filtro_entrante_saliente] = params[:filtro][:entrante_saliente]
    redirect_to :action => :listado
  end

  def elementos_filtrado
    @libro = Libro.find( session[:transferencia_filtro_libro] ) unless session[:transferencia_filtro_libro] == "todas"
    @etapa = Etapa.find( session[:transferencia_filtro_etapa] ) unless session[:transferencia_filtro_etapa] == "todas"
    @tipo = session[:transferencia_filtro_tipo].to_s unless session[:transferencia_filtro_tipo] == "todos"
    @entrante_saliente = session[:transferencia_filtro_entrante_saliente].to_s unless session[:transferencia_filtro_entrante_saliente] == "todas"

    filtro_etapa = [["Todas","todas"]] +   eval( "@" + singularizar_seccion ).etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    filtro_libro = [["Todas","todas"]] +   eval( "@" + singularizar_seccion ).libro.select{|l| session[:usuario_identificado].libro.include? l}.collect {|a|[a.nombre, a.id]}
    filtro_entrante_saliente = [["Todas","todas"], ["Entrante","entrante"],["Saliente","saliente"] ]

    @opciones_filtrado = [{:rotulo =>  _("Selecciona etapa: "), :nombre => "etapa", :opciones => filtro_etapa},
                          {:rotulo =>  _("Selecciona cuenta: "), :nombre => "libro", :opciones => filtro_libro},
                          {:rotulo =>  _("Selecciona tipo: "), :nombre => "tipo", :opciones => filtro_tipo },
                          {:rotulo =>  _("Entrante/Saliente: "), :nombre => "entrante_saliente", :opciones => filtro_entrante_saliente } ]

    @estado_filtrado = [(@etapa ? (@etapa.nombre + " (" + @etapa.fecha_inicio.to_s + "/" + @etapa.fecha_fin.to_s + ")") : "Cualquier etapa"),
                       (@libro ? @libro.nombre : "Cualquier cuenta/caja"),
                       (@tipo || "Cualquier tipo"),
                       (@entrante_saliente || "Todas (entrantes y salientes)") ]
    @resumen = {:url => {:action => :arqueo_caja, :controller => :resumen_proyecto, :sin_layout => true}, :mensaje => "Ver resumen de arqueo de caja"}
  end

  # en proyectos y en agente: lista
  def listado
    elementos_filtrado

    condiciones = { "libro.agente_id" => @agente.id } if params[:seccion] == "agentes"
    condiciones = { "proyecto_id" => @proyecto.id } if params[:seccion] == "proyectos"

    condiciones[:tipo] = @tipo if @tipo 
    condiciones[:libro_id] = @libro.id if @libro 
    condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin if @etapa
    condiciones[:entrante_saliente] = @entrante_saliente if @entrante_saliente

    @transferencias = Transferencia.paginate( :all, :conditions => condiciones,
                    :include => :libro,
                    :order => (session[:transferencia_cadena_orden] + " " + session[:transferencia_asc_desc] ),
                                :page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:transferencia_numero]) )

    @formato_xls = @transferencias.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "transferencia"
        @objetos = @transferencias
        render 'comunes_xls/listado', :layout => false
      end
    end

  end


	# en proyectos y en agente: prepara el formulario de edición o creación
  def editar_nuevo
    @transferencia = params[:id] ?  Transferencia.find(params[:id]) : Transferencia.new
    @etapa = Etapa.find( session[:transferencia_filtro_etapa] ) unless session[:transferencia_filtro_etapa] == "todas"
    @proyectos = @agente.proyecto_implementador.collect{ |p| [p.nombre, p.id] } if params[:seccion] == "agentes"
    @proyecto_asociado = @proyecto ? @proyecto : @transferencia.proyecto
    # Si proviene del formulario general y se ha elegido un tipo, pinta el formulario
    if params[:tipo]
      render :update do |page|
        page.replace "MB_caption", :inline => "<div id='MB_caption'>" + ( @proyecto ? "Proyecto: " + @proyecto.nombre : "Agente: " + @agente.nombre) + "<br>Nuevo movimiento: " + params[:tipo].capitalize + "</div>"
        page.replace params[:update], :partial => "formulario_" + params[:tipo]
        page.call("Modalbox.resizeToContent");
      end 
    # Si es uno nuevo, o ya existia el movimiento pinta el formulario correspondiente
    else
      tipos = filtro_tipo
      tipos.shift
      render :partial => "formulario" + ( @transferencia.id.nil? ? "" : "_" + @transferencia.tipo ), :locals => {:tipos => tipos}
    end
  end

  # en proyectos y en agente: modifica o crea
  def modificar_crear
    @etapa = Etapa.find( session[:transferencia_filtro_etapa] ) unless  session[:transferencia_filtro_etapa] == "todas"
    transferencia = params[:id] ?  Transferencia.find(params[:id]) : Transferencia.new 
    # Le asignamos el proyecto o el agente
    params[:transferencia][(singularizar_seccion + "_id").to_sym] = eval( "@" + singularizar_seccion ).id
    # Si es un movimiento saliente, no tiene tasa de cambio
    params[:transferencia][:tasa_cambio] = nil if params[:transferencia][:entrante_saliente] == "saliente"

    # Le metemos los datos del formulario, pero sin guardar 
    transferencia.attributes = params[:transferencia]
    # Actualizamos los datos del movimiento 
    transferencia.update_attributes params[:transferencia] if (session[:transferencia_filtro_etapa] == "todas" or transferencia.comprobar_fecha_etapa @etapa)

    # En el caso de que sea un movimiento nuevo y entrante y la cuenta destino sea tambien del usuario, generamos la salida 
    if params[:id].nil? && params[:transferencia][:entrante_saliente] == "entrante" && params[:selector][:entrada] == "1" &&
                params[:transferencia][:libro_receptor_emisor_id] && session[:usuario_identificado].libro.find_by_id(params[:transferencia][:libro_receptor_emisor_id])
      movimiento_vuelta = Transferencia.new()
      movimiento_vuelta.attributes = params[:transferencia]
      movimiento_vuelta.entrante_saliente = "saliente"
      movimiento_vuelta.tasa_cambio = nil
      movimiento_vuelta.libro_id = transferencia.libro_receptor_emisor_id
      movimiento_vuelta.libro_receptor_emisor_id =  transferencia.libro_id
      movimiento_vuelta.moneda_id = Libro.find_by_id( transferencia.libro_receptor_emisor_id ).moneda_id
      movimiento_vuelta.importe = transferencia.importe * transferencia.tasa_cambio if transferencia.importe && transferencia.tasa_cambio
    end if params[:transferencia][:tipo] =~ /transferencia|cambio|cheque|retirada/

    # Si no hemos tenido errores guardamos el movimiento de salida 
    if transferencia.errors.empty? && movimiento_vuelta
      movimiento_vuelta.save
      # Y vinculamos los 2 movimientos
      transferencia.update_attribute :transferencia_id, movimiento_vuelta.id
      movimiento_vuelta.update_attribute :transferencia_id, transferencia.id
      flash[:error] = movimiento_vuelta
    else
      flash[:error] = transferencia
    end

    redirect_to :action => "ordenado"
  end

  # en proyectos y en agente: elimina
  def eliminar
    Transferencia.find(params[:id]).destroy
    flash[:mensaje] = _('Se ha eliminado transferencia.') 
    redirect_to :action => 'ordenado'
  end

  # en proyectos y en agente: muestra la moneda del libro seleccionado
  def moneda_libro
    @moneda = params[:id] != "" ? Libro.find(params[:id]).moneda : nil
    render :inline => <<-FIN
      <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
      <%= hidden_field('transferencia', 'moneda_id', :value => @moneda.id) if @moneda%>
    FIN
  end

  # en proyectos y en agente: reacciona al cambio de libro
  def cambio_libro
    libro=Libro.find(params[:id]) if params[:id] != ""
    @moneda = libro ? libro.moneda : nil
    tipo_libro = libro.nil? ? "todos" : (libro.tipo == "banco" ? "caja \"chica\"" : "banco")
    @direccion = tipo_libro == "banco" ? params[:direccion][1] : params[:direccion][0]
    @proyecto_asociado = @proyecto ? @proyecto : @transferencia.proyecto
    render :update do |page|
      page.replace "moneda", :inline => <<-FIN
        <div class="elemento1_3" id="moneda">
          <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
          <%= hidden_field('transferencia', 'moneda_id', :value => @moneda.id) if @moneda %>
        </div>
      FIN
      page.replace "transferencia_entrante_saliente", :inline => <<-FIN
        <%= hidden_field('transferencia', 'entrante_saliente', :value => @direccion ) %>
      FIN
      page.hide "tasa_cambio" if @direccion == "saliente"
      page.show "tasa_cambio" unless @direccion == "saliente"
      page.replace "selector_segundo_libro", :partial => "selector_segundo_libro", :locals => { :tipo_libro => tipo_libro, :moneda => @moneda }
    end
  end

  def gastos
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      page.replace_html params[:update], :partial => "gastos", :locals => { :update => params[:update], :transferencia_id => params[:transferencia_id], :tipo => params[:tipo], :gastos => @gastos }
    end
  end

        # en proyectos y en financiación:  prepara el formulario de edición o creación de comentarios
  def asociar_gasto
    @gastos = @proyecto.gasto.collect{ |g| [g.importe.to_s + " -- " + g.partida.nombre + " -- " + (g.concepto || "") , g.id]}
    render :partial => "asociar_gasto", :locals => { :transferencia_id => params[:transferencia_id], :update => params[:update] }
  end

        # en proyectos y en financiación:  modifica o crea un comentario
  def guardar_asociar_gasto
    @gasto_x_transferencia = GastoXTransferencia.create( :gasto_id => params[:selector][:gasto], :transferencia_id => params[:transferencia_id] )
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      page.replace 'formulario', :inline => '<%= mensaje_error(@gasto_x_transferencia) %><br>'
      page.call("Modalbox.resizeToContent")
      page.replace_html params[:update], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos, :update => params[:update] }
      page.visual_effect :highlight, params[:update] , :duration => 6
    end
  end

        # en proyectos y en financiación:  elimina un comentario
  def eliminar_gasto
    @objeto = GastoXTransferencia.find(:first, :conditions => {:gasto_id => params[:gasto_id], :transferencia_id => params[:transferencia_id]})
    not @objeto.nil? and @objeto.destroy
    @gastos = GastoXTransferencia.find_all_by_transferencia_id( params[:transferencia_id] ).collect{ |gxt| Gasto.find(gxt.gasto_id) }
    render :update do |page|
      #page.replace "gastos_" + params[:transferencia_id], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos }
      page.replace_html params[:update], :partial => "gastos", :locals => { :transferencia_id => params[:transferencia_id], :gastos => @gastos, :update => params[:update]} 
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@objeto, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

  def cuenta_segunda_financiador
    @proyecto_asociado = Proyecto.find(params[:id])
    render :update do |page|
      page.replace_html "cuenta_segunda_financiador", :partial => "cuenta_segunda_financiador", :locals => { :filtro_tipo => params[:filtro_tipo] } 
      page.call("Modalbox.resizeToContent")
    end 
  end

 # --
 # VINCULACION DE MOVIMIENTOS
 # ++

  def vincular_movimientos
    @transferencia = Transferencia.find params[:id]
    condiciones = { "libro.agente_id" => @agente.id } if params[:seccion] == "agentes"
    condiciones = { "proyecto_id" => @proyecto.id } if params[:seccion] == "proyectos"
    condiciones[:transferencia_id] = nil
    condiciones[:entrante_saliente] = @transferencia.entrante_saliente == "entrante" ? "saliente" : "entrante"
    condiciones[:tipo] = @transferencia.tipo
    @transferencias = Transferencia.find(:all, :conditions => condiciones, :include => :libro, :order => " fecha asc ")
    @transferencias += [@transferencia.transferencia_vinculada] if @transferencia.transferencia_vinculada
    @transferencias = @transferencias.collect { |t| [t.fecha.to_s + "  //  " + t.importe_convertido + "  //  " + t.libro.nombre, t.id] }
    render :partial => "vincular_movimientos"
  end

  def guardar_vinculacion
    @transferencia = Transferencia.find params[:id]
    @transferencia.transferencia_vinculada.update_attribute :transferencia_id, nil if @transferencia.transferencia_vinculada
    if params[:transferencia][:desvincular] == "1"
      @transferencia.update_attribute :transferencia_id, nil
    else
      @transferencia.update_attribute :transferencia_id, params[:transferencia][:transferencia_id]
      @transferencia = Transferencia.find params[:transferencia][:transferencia_id]
      @transferencia.update_attribute :transferencia_id, params[:id]
    end
    redirect_to :action => "listado"
  end

  private
    def filtro_tipo
      [ ["Todos","todos"], ["Transferencia","transferencia"],["Cambio","cambio"],["Cheque","cheque"],["Retirada","retirada"],["Adelanto","adelanto"],
                        ["Ingreso","ingreso"],["Intereses","intereses"],["Subvencion","subvencion"] ]
    end
end
