#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
class Libro < ActiveRecord::Base

  untranslate_all
  belongs_to :agente
  belongs_to :moneda
  belongs_to :pais

  has_many :proyecto, :through => :libro_x_proyecto
  has_many :libro_x_proyecto, :dependent => :destroy
  has_many :libro_x_proyecto
  #has_many :gasto, :dependent => :nullify
  has_many :pago, :dependent => :nullify
  has_many :presupuesto, :dependent => :nullify
  has_many :usuario_x_libro, :dependent => :destroy
  has_many :usuario, :through => :usuario_x_libro
  has_many :transferencia

  validate :nombre_mayusculas
  validates_presence_of :nombre, :message => _("Nombre") + _(" no pueden estar vacío.")
  validates_presence_of :agente_id, :message => _("Agente") + _(" no pueden estar vacío.")
  validates_presence_of :moneda_id, :message => _("Moneda") + _(" no pueden estar vacío.")
  validates_presence_of :pais_id, :message => _("Pais") + _(" no pueden estar vacío.")
  validates_uniqueness_of :nombre, :message => _("Nombre") +   _(" repetido.")
  before_destroy :verificar_borrado

  def nombre_mayusculas
    self.nombre = self.nombre.upcase
  end

  # actualiza los usuario del libro (utilizado por LibroController::guardar_usuarios )
  def actualizar_usuario_x_libro listado
    usuario_x_libro.clear
    listado.each do |key, value|
        uxp = usuario_x_libro.create(value)
        errors.add("", uxp.errors.inject('') {|total, e| total + e[1] + "<br>" }) unless uxp.errors.empty?
    end        
  end

  def verificar_borrado
    #errors.add( "gasto", _("hay gastos")) unless self.gasto.empty?
    errors.add( "pago", _("hay pagos asociados")) unless self.pago.empty?
    errors.add( "transferencia", _("hay transferencias asociadas")) unless self.transferencia.empty?
    errors.add_to_base( _("Una cuenta tiene que estar vacio para poder ser borrado.") ) unless errors.empty?
    return false unless errors.empty?
 end

end
